@@ -33,13 +33,6 @@ class SlOSCThread(nsm_client.NSMThread):
 
         if general_object.wait_for_load:
             general_object.sl_ready.emit()
-        
-        if jack_client is not None:
-            self.send(sl_url, '/register_update', 'loop_add', self.url, '/loop_add')  
-            
-    @make_method('/loop_add', 'if')
-    def loopAdd(self, path, args):
-        print('gjirjiji', args)
 
 
 class GeneralObject(QObject):
@@ -52,12 +45,9 @@ class GeneralObject(QObject):
         self.sl_process.setProcessChannelMode(QProcess.ForwardedChannels)
         self.sl_process.finished.connect(self.slProcessFinished)
 
-        self.sl_port = ray.getFreeOscPort(9951)
+        self.sl_port = ray.get_free_osc_port(9951)
         self.sl_url = Address(self.sl_port)
 
-
-        self.sl_process.start('sooperlooper', ['-p', str(self.sl_port)])
-
         self.gui_process = QProcess()
         self.gui_process.setProcessChannelMode(QProcess.ForwardedChannels)
         self.gui_process.started.connect(self.guiProcessStarted)
@@ -83,11 +73,10 @@ class GeneralObject(QObject):
 
         self.sl_ready.connect(self.loadSession)
 
+        self._switching = False
         self.leaving = False
         self.wait_for_load = False
 
-        #self.showOptionalGui()
-
         self.ping_timer = QTimer()
         self.ping_timer.setInterval(100)
         self.ping_timer.timeout.connect(self.pingSL)
@@ -99,6 +88,8 @@ class GeneralObject(QObject):
 
         self.transport_playing = False
         self.will_trig = False
+        
+        self.jack_follow_naming = False
 
     def JackShutdownCallback(self, arg=None):
         self.transport_timer.stop()
@@ -159,7 +150,8 @@ class GeneralObject(QObject):
         return bool(self.sl_process.state() == QProcess.Running)
 
     def slProcessFinished(self, exit_code):
-        app.quit()
+        if not self._switching:
+            app.quit()
 
     def guiProcessStarted(self):
         server.sendGuiState(True)
@@ -250,12 +242,35 @@ class GeneralObject(QObject):
     def initialize(self, project_path, session_name, full_client_id):
         self.project_path = project_path
         self.session_name = session_name
-        self.full_client_id = full_client_id
         self.session_file = "%s/session.slsess" % self.project_path
         self.session_bak = "%s/session.slsess.bak" % self.project_path
         self.midi_bindings_file = "%s/session.slb" % self.project_path
         #self.midi_bindings_bak = "%s/session.slb.bak" % self.project_path
 
+        if not self.jack_follow_naming:
+            full_client_id = 'sooperlooper'
+
+        if full_client_id != self.full_client_id:
+            self.full_client_id = full_client_id
+
+            if self.gui_process.state():
+                self.gui_process.terminate()
+                self.gui_process.waitForFinished(500)
+            else:
+                server.sendGuiState(False)
+            
+            self._switching = True
+            
+            if self.sl_process.state():
+                self.sl_process.terminate()
+                self.sl_process.waitForFinished(500)
+            
+            self._switching = False
+            
+            self.sl_process.start(
+                'sooperlooper',
+                ['-p', str(self.sl_port), '-j', self.full_client_id])
+
         if not os.path.exists(self.project_path):
             os.makedirs(self.project_path)
 
@@ -271,6 +286,7 @@ class GeneralObject(QObject):
             self.transport_timer.start()
 
     def loadSession(self):
+        #self.sl_process.start('sooperlooper', ['-p', str(self.sl_port)])
         self.wait_for_load = False
         server.send(self.sl_url, '/load_session', self.session_file,
                     server.url, '/re-load')
@@ -312,7 +328,7 @@ if __name__ == '__main__':
         sys.stderr.write('Could not register as NSM client.\n')
         sys.exit()
 
-    daemon_address = ray.getLibloAddress(NSM_URL)
+    daemon_address = ray.get_liblo_address(NSM_URL)
 
     signal.signal(signal.SIGINT, signalHandler)
     signal.signal(signal.SIGTERM, signalHandler)
@@ -330,7 +346,7 @@ if __name__ == '__main__':
 
     server = SlOSCThread('sooperlooper_nsm', signaler, daemon_address, False)
 
-    if len(sys.argv) > 1 and sys.argv[1] == '--transport_workaround':
+    if len(sys.argv) > 1 and '--transport_workaround' in sys.argv[1:]:
         jack_client = jacklib.client_open(
             "sooper_ray_wk",
             jacklib.JackNoStartServer | jacklib.JackSessionID,
@@ -339,9 +355,13 @@ if __name__ == '__main__':
         jack_client = None
 
     general_object = GeneralObject()
+    if "--follow-jack-naming" in sys.argv[1:]:
+        general_object.jack_follow_naming = True
 
     server.start()
-    server.announce('SooperLooper', ':optional-gui:switch:', 'sooperlooper_nsm')
+    
+    capabilities = ':optional-gui:switch:'
+    server.announce('SooperLooper', capabilities, 'sooperlooper_nsm')
 
     app.exec()
 
