@@ -30,17 +30,6 @@ server_operations = (
     'rename_session', 'set_options', 'has_option',
     'script_info', 'hide_script_info', 'script_user_action')
 
-session_operations = ('save', 'save_as_template', 'take_snapshot',
-                      'close', 'abort', 'duplicate', 'open_snapshot',
-                      'rename', 'set_notes', 'get_notes',
-                      'add_executable', 'add_proxy',
-                      'add_factory_client_template',
-                      'add_user_client_template',
-                      'add_client_template', 'list_snapshots',
-                      'list_clients', 'list_trashed_clients',
-                      'reorder_clients',
-                      'get_session_name', 'run_step', 'clear_clients')
-
 
 def signalHandler(sig, frame):
     if sig in (signal.SIGINT, signal.SIGTERM):
@@ -55,7 +44,7 @@ def addSelfBinToPath():
     if not os.environ['PATH'].startswith("%s:" % bin_path):
         os.environ['PATH'] = "%s:%s" % (bin_path, os.environ['PATH'])
 
-def pidExists(pid):
+def pidExists(pid: int)->bool:
     if isinstance(pid, str):
         pid = int(pid)
 
@@ -66,6 +55,18 @@ def pidExists(pid):
     else:
         return True
 
+def pidIsStopped(pid: int)->bool:
+    proc_file_path = '/proc/%i/status' % pid
+    if os.path.exists(proc_file_path):
+        proc_file = open(proc_file_path)
+        for line in proc_file.readlines():
+            if line.startswith('State:	'):
+                value = line.replace('State:	', '', 1)
+                if value and value[0] == 'T':
+                    return True
+        return False
+    return True
+
 def getDaemonList():
     try:
         tree = ET.parse('/tmp/RaySession/multi-daemon.xml')
@@ -105,6 +106,12 @@ def getDaemonList():
             elif key == 'has_gui':
                 l_daemon.has_local_gui = bool(child.attrib[key] == '3')
                 l_daemon.has_gui = bool(child.attrib[key] == '1')
+                
+            elif key == 'local_gui_pids':
+                gui_pids_str = child.attrib[key]
+                for pid_str in gui_pids_str.split(':'):
+                    if pid_str.isdigit():
+                        l_daemon.local_gui_pids.append(int(pid_str))
 
         if not (l_daemon.net_daemon_id
                 and l_daemon.pid
@@ -124,6 +131,9 @@ class Daemon:
     not_default = False
     has_gui = 0
     has_local_gui = 0
+    
+    def __init__(self):
+        self.local_gui_pids = []
 
 
 def printHelp(stdout=False, category=OPERATION_TYPE_NULL):
@@ -228,6 +238,9 @@ if __name__ == '__main__':
 
         elif option == '--detach':
             detach = True
+        else:
+            printHelp()
+            sys.exit(100)
 
     operation = args.pop(0)
     if operation in ('client', 'trashed_client'):
@@ -247,12 +260,8 @@ if __name__ == '__main__':
             operation_type = OPERATION_TYPE_CONTROL
         elif operation in server_operations:
             operation_type = OPERATION_TYPE_SERVER
-        elif operation in session_operations:
-            operation_type = OPERATION_TYPE_SESSION
         else:
-            sys.stderr.write("Unknown operation: %s\n" % operation)
-            printHelp()
-            sys.exit(100)
+            operation_type = OPERATION_TYPE_SESSION
 
     arg_list = [autoTypeString(s) for s in args]
     if operation_type in (OPERATION_TYPE_CLIENT,
@@ -321,14 +330,32 @@ if __name__ == '__main__':
                 sys.exit(0)
 
             elif operation == 'get_port_gui_free':
+                wanted_session_root = ''
+                if args:
+                    wanted_session_root = args[0]
+
                 for daemon in daemon_list:
                     if (daemon.user == os.environ['USER']
-                            and not daemon.not_default
-                            and not daemon.has_local_gui):
-                        sys.stdout.write('%s\n' % daemon.port)
-                        break
-                sys.exit(0)
+                            and (daemon.root == wanted_session_root
+                                 or not wanted_session_root)
+                            and not daemon.not_default):
+                        if not daemon.has_local_gui:
+                            sys.stdout.write('%s\n' % daemon.port)
+                            break
 
+                        for pid in daemon.local_gui_pids:
+                            if pid == 0:
+                                # This means we don't know the pid of the local GUI
+                                # So consider this daemon has already a GUI
+                                break
+
+                            if pidExists(pid) and not pidIsStopped(pid):
+                                break
+                        else:
+                            sys.stdout.write('%s\n' % daemon.port)
+                            break
+
+                sys.exit(0)
 
             elif operation == 'get_root':
                 for daemon in daemon_list:
@@ -430,11 +457,12 @@ if __name__ == '__main__':
     else:
         session_root = "%s/Ray Sessions" % os.getenv('HOME')
         try:
-            settings_file = open("%s/.config/RaySession/RaySession.conf", 'r')
+            settings_file = open(
+                "%s/.config/RaySession/RaySession.conf" % os.getenv('HOME'), 'r')
             contents = settings_file.read()
             for line in contents.split('\n'):
                 if line.startswith('default_session_root='):
-                    session_root = line.replace('default_session_root', '', 1)
+                    session_root = line.partition('=')[2]
                     break
         except:
             pass
