@@ -7,7 +7,7 @@ import sys
 from PyQt5.QtCore import QSettings, QDataStream, QIODevice, QUrl, QByteArray
 from PyQt5.QtXml  import QDomDocument
 import ray
-from daemon_tools import getAppConfigPath
+from daemon_tools import get_app_config_path, dirname
 
 QFileDialogMagic = 190
 
@@ -16,13 +16,7 @@ class PickerType:
         self.config_path = config_path
         self.written = False
 
-    def makeBookmark(self, spath):
-        pass
-
-    def removeBookmark(self, spath):
-        pass
-
-    def getContents(self):
+    def _get_contents(self):
         if os.path.exists(self.config_path):
             try:
                 file = open(self.config_path, 'r')
@@ -34,7 +28,7 @@ class PickerType:
         else:
             return ""
 
-    def printContents(self, contents):
+    def _print_contents(self, contents):
         try:
             file = open(self.config_path, 'w')
         except:
@@ -45,14 +39,20 @@ class PickerType:
 
         return True
 
+    def make_bookmark(self, spath):
+        pass
+
+    def remove_bookmark(self, spath):
+        pass
+
 class PickerTypeGtk(PickerType):
-    def makeBookmark(self, spath):
+    def make_bookmark(self, spath):
         if self.written:
             return
 
         url = pathlib.Path(spath).as_uri()
 
-        config_dir = os.path.dirname(self.config_path)
+        config_dir = dirname(self.config_path)
 
         if not os.path.exists(config_dir):
             try:
@@ -60,7 +60,7 @@ class PickerTypeGtk(PickerType):
             except:
                 return
 
-        contents = self.getContents()
+        contents = self._get_contents()
         if contents is None:
             return
 
@@ -71,16 +71,16 @@ class PickerTypeGtk(PickerType):
 
         contents += "%s\n" % url
 
-        if self.printContents(contents):
+        if self._print_contents(contents):
             self.written = True
 
-    def removeBookmark(self, spath):
+    def remove_bookmark(self, spath):
         if not self.written:
             return
 
         url = pathlib.Path(spath).as_uri()
 
-        contents = self.getContents()
+        contents = self._get_contents()
         if not contents:
             self.written = False
             return
@@ -98,15 +98,15 @@ class PickerTypeGtk(PickerType):
             if url:
                 contents += "%s\n" % url
 
-        self.printContents(contents)
+        self._print_contents(contents)
         self.written = False
 
 class PickerTypeFltk(PickerType):
-    def makeBookmark(self, spath):
+    def make_bookmark(self, spath):
         if self.written:
             return
 
-        contents = self.getContents()
+        contents = self._get_contents()
         if not contents:
             #we won't write a file for fltk if file doesn't already exists
             return
@@ -141,14 +141,14 @@ class PickerTypeFltk(PickerType):
             num += 1
             contents += "favorite%.2d:%s" % (num, spath)
 
-        if self.printContents(contents):
+        if self._print_contents(contents):
             self.written = True
 
-    def removeBookmark(self, spath):
+    def remove_bookmark(self, spath):
         if not self.written:
             return
 
-        contents = self.getContents()
+        contents = self._get_contents()
         if not contents:
             self.written = False
             return
@@ -180,11 +180,11 @@ class PickerTypeFltk(PickerType):
             else:
                 contents += "%s\n" % line
 
-        self.printContents(contents)
+        self._print_contents(contents)
         self.written = False
 
 class PickerTypeQt4(PickerType):
-    def makeBookmark(self, spath):
+    def make_bookmark(self, spath):
         if self.written:
             return
 
@@ -256,7 +256,7 @@ class PickerTypeQt4(PickerType):
 
         self.written = True
 
-    def removeBookmark(self, spath):
+    def remove_bookmark(self, spath):
         if not self.written:
             return
 
@@ -334,7 +334,7 @@ class PickerTypeQt4(PickerType):
         self.written = False
 
 class PickerTypeQt5(PickerType):
-    def makeBookmark(self, spath):
+    def make_bookmark(self, spath):
         if self.written:
             return
 
@@ -348,7 +348,8 @@ class PickerTypeQt5(PickerType):
         if not settings_qt5.isWritable():
             return
 
-        shortcuts = ray.getListInSettings(settings_qt5, 'FileDialog/shortcuts')
+        shortcuts = ray.get_list_in_settings(settings_qt5,
+                                             'FileDialog/shortcuts')
 
         for sc in shortcuts:
             sc_url = QUrl(sc)
@@ -361,7 +362,7 @@ class PickerTypeQt5(PickerType):
         settings_qt5.sync()
         self.written = True
 
-    def removeBookmark(self, spath):
+    def remove_bookmark(self, spath):
         if not self.written:
             return
 
@@ -370,8 +371,8 @@ class PickerTypeQt5(PickerType):
             return
 
         settings_qt5 = QSettings(self.config_path, QSettings.IniFormat)
-        shortcuts = ray.getListInSettings(settings_qt5,
-                                          'FileDialog/shortcuts')
+        shortcuts = ray.get_list_in_settings(settings_qt5,
+                                             'FileDialog/shortcuts')
 
         for sc in shortcuts:
             sc_url = QUrl(sc)
@@ -387,11 +388,11 @@ class PickerTypeQt5(PickerType):
         self.written = False
 
 class PickerTypeKde5(PickerType):
-    def makeBookmark(self, spath):
+    def make_bookmark(self, spath):
         if self.written:
             return
 
-        contents = self.getContents()
+        contents = self._get_contents()
         if not contents:
             #we won't write a file for kde5 if file doesn't already exists
             return
@@ -422,15 +423,15 @@ class PickerTypeKde5(PickerType):
         bk.appendChild(title)
         content.appendChild(bk)
 
-        if self.printContents(xml.toString()):
+        if self._print_contents(xml.toString()):
             self.written = True
 
 
-    def removeBookmark(self, spath):
+    def remove_bookmark(self, spath):
         if not self.written:
             return
 
-        contents = self.getContents()
+        contents = self._get_contents()
         if not contents:
             self.written = False
             return
@@ -457,40 +458,37 @@ class PickerTypeKde5(PickerType):
             self.written = False
             return
 
-        self.printContents(xml.toString())
+        self._print_contents(xml.toString())
         self.written = False
 
 
 class BookMarker:
     def __init__(self):
-        self.bookmarks_memory = "%s/bookmarks.xml" % getAppConfigPath()
-        self.daemon_port = 0
+        self._bookmarks_memory = "%s/bookmarks.xml" % get_app_config_path()
+        self._daemon_port = 0
 
-        HOME = os.getenv('HOME')
+        home = os.getenv('HOME')
 
-        self.gtk2 = PickerTypeGtk("%s/.gtk-bookmarks" % HOME)
-        self.gtk3 = PickerTypeGtk("%s/.config/gtk-3.0/bookmarks" % HOME)
-        self.fltk = PickerTypeFltk("%s/.fltk/fltk.org/filechooser.prefs"
-                                   % HOME)
-        self.kde5 = PickerTypeKde5("%s/.local/share/user-places.xbel" % HOME)
-        self.qt4 = PickerTypeQt4("%s/.config/Trolltech.conf" % HOME)
-        self.qt5 = PickerTypeQt5("%s/.config/QtProject.conf" % HOME)
+        self._gtk2 = PickerTypeGtk("%s/.gtk-bookmarks" % home)
+        self._gtk3 = PickerTypeGtk("%s/.config/gtk-3.0/bookmarks" % home)
+        self._fltk = PickerTypeFltk(
+            "%s/._fltk/fltk.org/filechooser.prefs" % home)
+        self._kde5 = PickerTypeKde5("%s/.local/share/user-places.xbel" % home)
+        self._qt4 = PickerTypeQt4("%s/.config/Trolltech.conf" % home)
+        self._qt5 = PickerTypeQt5("%s/.config/QtProject.conf" % home)
 
-    def setDaemonPort(self, port):
-        self.daemon_port = port
-
-    def getXml(self):
+    def _get_xml(self):
         xml = QDomDocument()
         file_exists = False
 
-        if os.path.exists(self.bookmarks_memory):
+        if os.path.exists(self._bookmarks_memory):
             try:
-                file = open(self.bookmarks_memory, 'r')
+                file = open(self._bookmarks_memory, 'r')
                 xml.setContent(file.read())
                 file_exists = True
             except:
                 try:
-                    os.path.remove(self.bookmarks_memory)
+                    os.path.remove(self._bookmarks_memory)
                 except:
                     return None
 
@@ -500,56 +498,59 @@ class BookMarker:
 
         return xml
 
-    def writeXmlFile(self, xml):
+    def _write_xml_file(self, xml):
         try:
-            file = open(self.bookmarks_memory, 'w')
+            file = open(self._bookmarks_memory, 'w')
             file.write(xml.toString())
         except:
             return
 
-    def getPickersForXml(self):
+    def _get_pickers_for_xml(self):
         string = ":"
-        if self.gtk2.written:
+        if self._gtk2.written:
             string += "gtk2:"
-        if self.gtk3.written:
+        if self._gtk3.written:
             string += "gtk3:"
-        if self.fltk.written:
+        if self._fltk.written:
             string += "fltk:"
-        if self.kde5.written:
+        if self._kde5.written:
             string += "kde5:"
-        if self.qt4.written:
+        if self._qt4.written:
             string += "qt4:"
-        if self.qt5.written:
+        if self._qt5.written:
             string += "qt5:"
 
         return string
 
-    def makeAll(self, spath):
-        for picker in (self.gtk2, self.gtk3, self.fltk,
-                       self.kde5, self.qt4, self.qt5):
-            picker.makeBookmark(spath)
+    def set_daemon_port(self, port):
+        self._daemon_port = port
 
-        xml = self.getXml()
+    def make_all(self, spath):
+        for picker in (self._gtk2, self._gtk3, self._fltk,
+                       self._kde5, self._qt4, self._qt5):
+            picker.make_bookmark(spath)
+
+        xml = self._get_xml()
         if not xml:
             return
 
         xml_content = xml.documentElement()
 
         bke = xml.createElement('bookmarker')
-        bke.setAttribute('port', self.daemon_port)
+        bke.setAttribute('port', self._daemon_port)
         bke.setAttribute('session_path', spath)
-        bke.setAttribute('pickers', self.getPickersForXml())
+        bke.setAttribute('pickers', self._get_pickers_for_xml())
 
         xml_content.appendChild(bke)
 
-        self.writeXmlFile(xml)
+        self._write_xml_file(xml)
 
-    def removeAll(self, spath):
-        for picker in (self.gtk2, self.gtk3, self.fltk,
-                       self.kde5, self.qt4, self.qt5):
-            picker.removeBookmark(spath)
+    def remove_all(self, spath):
+        for picker in (self._gtk2, self._gtk3, self._fltk,
+                       self._kde5, self._qt4, self._qt5):
+            picker.remove_bookmark(spath)
 
-        xml = self.getXml()
+        xml = self._get_xml()
         if not xml:
             return
 
@@ -563,16 +564,16 @@ class BookMarker:
             session_path = bke.attribute('session_path')
 
             if (port.isdigit()
-                    and int(port) == self.daemon_port
+                    and int(port) == self._daemon_port
                     and session_path == spath):
                 xml_content.removeChild(node)
                 break
 
-        self.writeXmlFile(xml)
+        self._write_xml_file(xml)
 
 
     def clean(self, all_session_paths):
-        xml = self.getXml()
+        xml = self._get_xml()
         if not xml:
             return
 
@@ -592,31 +593,31 @@ class BookMarker:
 
             if not spath in all_session_paths:
                 if ":gtk2:" in pickers:
-                    self.gtk2.written = True
-                    self.gtk2.removeBookmark(spath)
+                    self._gtk2.written = True
+                    self._gtk2.remove_bookmark(spath)
                 if ":gtk3:" in pickers:
-                    self.gtk3.written = True
-                    self.gtk3.removeBookmark(spath)
+                    self._gtk3.written = True
+                    self._gtk3.remove_bookmark(spath)
                 if ":fltk:" in pickers:
-                    self.fltk.written = True
-                    self.fltk.removeBookmark(spath)
+                    self._fltk.written = True
+                    self._fltk.remove_bookmark(spath)
                 if ":kde5:" in pickers:
-                    self.kde5.written = True
-                    self.kde5.removeBookmark(spath)
+                    self._kde5.written = True
+                    self._kde5.remove_bookmark(spath)
                 if ":qt4:" in pickers:
-                    self.qt4.written = True
-                    self.qt4.removeBookmark(spath)
+                    self._qt4.written = True
+                    self._qt4.remove_bookmark(spath)
                 if ":qt5:" in pickers:
-                    self.qt5.written = True
-                    self.qt5.removeBookmark(spath)
+                    self._qt5.written = True
+                    self._qt5.remove_bookmark(spath)
 
                 nodes_to_remove.append(node)
 
         for node in nodes_to_remove:
             xml_content.removeChild(node)
 
-        self.writeXmlFile(xml)
+        self._write_xml_file(xml)
 
 if __name__ == '__main__':
     bm_maker = BookMarker()
-    bm_maker.makeAll(sys.argv[1])
+    bm_maker.make_all(sys.argv[1])
