@@ -1,5 +1,5 @@
 import os
-from PyQt5.QtXml import QDomDocument
+from PyQt5.QtXml import QDomDocument, QDomElement
 
 import ray
 
@@ -21,16 +21,18 @@ class MultiDaemonFile:
         self.session = session
         self.server = server
 
-        self.xml = QDomDocument()
+        self._xml = QDomDocument()
 
         global instance
         instance = self
 
+        self._locked_session_paths = set()
+
     @staticmethod
-    def getInstance():
+    def get_instance():
         return instance
 
-    def pidExists(self, pid):
+    def _pid_exists(self, pid)->bool:
         if isinstance(pid, str):
             pid = int(pid)
 
@@ -41,13 +43,13 @@ class MultiDaemonFile:
         else:
             return True
 
-    def removeFile(self):
+    def _remove_file(self):
         try:
             os.remove(self.file_path)
         except:
             return
 
-    def openFile(self):
+    def _open_file(self)->bool:
         if not os.path.exists(self.file_path):
             dir_path = os.path.dirname(self.file_path)
             if not os.path.exists(dir_path):
@@ -59,23 +61,23 @@ class MultiDaemonFile:
 
         try:
             file = open(self.file_path, 'r')
-            self.xml.setContent(file.read())
+            self._xml.setContent(file.read())
             file.close()
             return True
 
         except:
-            self.removeFile()
+            self._remove_file()
             return False
 
-    def writeFile(self):
+    def _write_file(self):
         try:
             file = open(self.file_path, 'w')
-            file.write(self.xml.toString())
+            file.write(self._xml.toString())
             file.close()
         except:
             return
 
-    def setAttributes(self, element):
+    def _set_attributes(self, element):
         element.setAttribute('net_daemon_id', self.server.net_daemon_id)
         element.setAttribute('root', self.session.root)
         element.setAttribute('session_path', self.session.path)
@@ -83,54 +85,77 @@ class MultiDaemonFile:
         element.setAttribute('port', self.server.port)
         element.setAttribute('user', os.getenv('USER'))
         element.setAttribute('not_default',
-            int(bool(self.server.is_nsm_locked or self.server.not_default)))
-        element.setAttribute('has_gui', int(self.server.hasGui()))
+                             int(bool(self.server.is_nsm_locked
+                                      or self.server.not_default)))
+        element.setAttribute('has_gui', int(self.server.has_gui()))
         element.setAttribute('version', ray.VERSION)
-        element.setAttribute('local_gui_pids', self.server.getLocalGuiPidList())
+        element.setAttribute('local_gui_pids',
+                             self.server.get_local_gui_pid_list())
+
+        for locked_path in self._locked_session_paths:
+            locked_paths_xml = self._xml.createElement('locked_session')
+            locked_paths_xml.setAttribute('path', locked_path)
+            element.appendChild(locked_paths_xml)
+
+    def _clean_dirty_pids(self):
+        xml_content = self._xml.documentElement()
+        nodes = xml_content.childNodes()
+        rm_nodes = []
+
+        for i in range(nodes.count()):
+            node = nodes.at(i)
+            dxe = node.toElement()
+            pid = dxe.attribute('pid')
+            if not pid.isdigit() or not self._pid_exists(int(pid)):
+                rm_nodes.append(node)
+
+        for node in rm_nodes:
+            xml_content.removeChild(node)
 
     def update(self):
         has_dirty_pid = False
 
-        if not self.openFile():
-            ds = self.xml.createElement('Daemons')
-            dm_xml = self.xml.createElement('Daemon')
+        if not self._open_file():
+            ds = self._xml.createElement('Daemons')
+            dm_xml = self._xml.createElement('Daemon')
 
-            self.setAttributes(dm_xml)
+            self._set_attributes(dm_xml)
 
             ds.appendChild(dm_xml)
-            self.xml.appendChild(ds)
+            self._xml.appendChild(ds)
 
         else:
-            found = False
-            xml_content = self.xml.documentElement()
-
+            xml_content = self._xml.documentElement()
             nodes = xml_content.childNodes()
+            self_node = None
+
             for i in range(nodes.count()):
                 node = nodes.at(i)
                 dxe = node.toElement()
                 pid = dxe.attribute('pid')
 
                 if pid.isdigit() and pid == str(os.getpid()):
-                    self.setAttributes(dxe)
-                    found = True
-                elif not pid.isdigit() or not self.pidExists(int(pid)):
+                    self_node = node
+                elif not pid.isdigit() or not self._pid_exists(int(pid)):
                     has_dirty_pid = True
+            
+            if self_node is not None:
+                xml_content.removeChild(self_node)
 
-            if not found:
-                dm_xml = self.xml.createElement('Daemon')
-                self.setAttributes(dm_xml)
-                self.xml.firstChild().appendChild(dm_xml)
+            dm_xml = self._xml.createElement('Daemon')
+            self._set_attributes(dm_xml)
+            self._xml.firstChild().appendChild(dm_xml)
 
         if has_dirty_pid:
-            self.cleanDirtyPids()
+            self._clean_dirty_pids()
 
-        self.writeFile()
+        self._write_file()
 
     def quit(self):
-        if not self.openFile():
+        if not self._open_file():
             return
 
-        xml_content = self.xml.documentElement()
+        xml_content = self._xml.documentElement()
         nodes = xml_content.childNodes()
 
         for i in range(nodes.count()):
@@ -144,13 +169,13 @@ class MultiDaemonFile:
             return
 
         xml_content.removeChild(node)
-        self.writeFile()
+        self._write_file()
 
-    def isFreeForRoot(self, daemon_id, root_path):
-        if not self.openFile():
+    def is_free_for_root(self, daemon_id, root_path)->bool:
+        if not self._open_file():
             return True
 
-        xml_content = self.xml.documentElement()
+        xml_content = self._xml.documentElement()
         nodes = xml_content.childNodes()
 
         for i in range(nodes.count()):
@@ -159,35 +184,44 @@ class MultiDaemonFile:
             if (dxe.attribute('net_daemon_id') == str(daemon_id)
                     and dxe.attribute('root') == root_path):
                 pid = dxe.attribute('pid')
-                if pid.isdigit() and self.pidExists(int(pid)):
+                if pid.isdigit() and self._pid_exists(int(pid)):
                     return False
 
         return True
 
-    def isFreeForSession(self, session_path):
-        if not self.openFile():
+    def is_free_for_session(self, session_path)->bool:
+        if not self._open_file():
             return True
 
-        xml_content = self.xml.documentElement()
+        xml_content = self._xml.documentElement()
         nodes = xml_content.childNodes()
 
         for i in range(nodes.count()):
             node = nodes.at(i)
             dxe = node.toElement()
-            if dxe.attribute('session_path') == session_path:
-                pid = dxe.attribute('pid')
-                if pid.isdigit() and self.pidExists(int(pid)):
+            pid = dxe.attribute('pid')
+
+            if dxe.attribute('session_path') == session_path:                
+                if pid.isdigit() and self._pid_exists(int(pid)):
                     return False
+            
+            sub_nodes = node.childNodes()
+            for j in range(sub_nodes.count()):
+                sub_node = sub_nodes.at(j)
+                sub_nod_el = sub_node.toElement()
+                if sub_nod_el.attribute('path') == session_path:
+                    if pid.isdigit() and self._pid_exists(int(pid)):
+                        return False
 
         return True
 
-    def getAllSessionPaths(self):
-        if not self.openFile():
+    def get_all_session_paths(self)->list:
+        if not self._open_file():
             return []
 
         all_session_paths = []
 
-        xml_content = self.xml.documentElement()
+        xml_content = self._xml.documentElement()
         nodes = xml_content.childNodes()
 
         for i in range(nodes.count()):
@@ -195,34 +229,27 @@ class MultiDaemonFile:
             dxe = node.toElement()
             spath = dxe.attribute('session_path')
             pid = dxe.attribute('pid')
-            if spath and pid.isdigit() and self.pidExists(int(pid)):
+            if spath and pid.isdigit() and self._pid_exists(int(pid)):
                 all_session_paths.append(spath)
 
         return all_session_paths
 
-    def cleanDirtyPids(self):
-        xml_content = self.xml.documentElement()
-        nodes = xml_content.childNodes()
-        rm_nodes = []
+    def add_locked_path(self, path:str):
+        self._locked_session_paths.add(path)
+        self.update()
+    
+    def unlock_path(self, path:str):
+        self._locked_session_paths.discard(path)
+        self.update()
 
-        for i in range(nodes.count()):
-            node = nodes.at(i)
-            dxe = node.toElement()
-            pid = dxe.attribute('pid')
-            if not pid.isdigit() or not self.pidExists(int(pid)):
-                rm_nodes.append(node)
-
-        for node in rm_nodes:
-            xml_content.removeChild(node)
-
-    def getDaemonList(self):
+    def get_daemon_list(self)->list:
         daemon_list = []
         has_dirty_pid = False
 
-        if not self.openFile():
+        if not self._open_file():
             return daemon_list
 
-        xml_content = self.xml.documentElement()
+        xml_content = self._xml.documentElement()
         nodes = xml_content.childNodes()
 
         for i in range(nodes.count()):
@@ -245,7 +272,7 @@ class MultiDaemonFile:
             if port.isdigit():
                 daemon.port = port
 
-            if not self.pidExists(daemon.pid):
+            if not self._pid_exists(daemon.pid):
                 has_dirty_pid = True
                 continue
 
@@ -257,6 +284,6 @@ class MultiDaemonFile:
             daemon_list.append(daemon)
 
         if has_dirty_pid:
-            self.cleanDirtyPids()
+            self._clean_dirty_pids()
 
         return daemon_list
