@@ -1,22 +1,23 @@
 import os
 import sys
 import time
+import subprocess
 
 from PyQt5.QtWidgets import (
     QDialog, QDialogButtonBox, QTreeWidget, QTreeWidgetItem,
-    QCompleter, QMessageBox, QFileDialog)
+    QCompleter, QMessageBox, QFileDialog, QApplication, QListWidgetItem)
 from PyQt5.QtGui import QIcon, QPixmap, QGuiApplication
 from PyQt5.QtCore import Qt, QTimer
 
 import client_properties_dialog
 import ray
 from gui_server_thread import GuiServerThread
-from gui_tools import (ErrDaemon, _translate,
+from gui_tools import (ErrDaemon, _translate, get_app_icon,
                        CommandLineArgs, RS, is_dark_theme)
 
 from patchcanvas import patchcanvas
 
-import ui.open_session
+
 import ui.new_session
 import ui.save_template_session
 import ui.nsm_open_info
@@ -40,6 +41,9 @@ import ui.snapshot_progress
 import ui.waiting_close_user
 import ui.client_rename
 import ui.canvas_port_info
+import ui.startup_dialog
+import ui.systray_close
+import ui.systray_management
 
 class ChildDialog(QDialog):
     def __init__(self, parent):
@@ -55,9 +59,7 @@ class ChildDialog(QDialog):
         self._root_folder_message_box = QMessageBox(
             QMessageBox.Critical,
             _translate('root_folder_dialogs', 'unwritable dir'),
-            '',
-            QMessageBox.NoButton,
-            self)
+            '', QMessageBox.NoButton, self)
 
         self.server_copying = parent.server_copying
 
@@ -134,279 +136,6 @@ class ChildDialog(QDialog):
         QDialog.enterEvent(self, event)
 
 
-class SessionItem(QTreeWidgetItem):
-    def __init__(self, l_list):
-        QTreeWidgetItem.__init__(self, l_list)
-
-    def __lt__(self, other):
-        if self.childCount() and not other.childCount():
-            return True
-
-        if other.childCount() and not self.childCount():
-            return False
-
-        return bool(self.text(0).lower() < other.text(0).lower())
-
-    def show_conditionnaly(self, string: str)->bool:
-        show = bool(string.lower() in self.data(0, Qt.UserRole).lower())
-
-        n = 0
-        for i in range(self.childCount()):
-            if self.child(i).show_conditionnaly(string.lower()):
-                n += 1
-        if n:
-            show = True
-
-        self.setExpanded(bool(n and string))
-        self.setHidden(not show)
-        return show
-
-    def find_item_with(self, string):
-        if self.data(0, Qt.UserRole) == string:
-            return self
-
-        item = None
-
-        for i in range(self.childCount()):
-            item = self.child(i).find_item_with(string)
-            if item:
-                break
-
-        return item
-
-class SessionFolder:
-    name = ""
-    has_session = True
-    path = ""
-
-    def __init__(self, name):
-        self.name = name
-        self.subfolders = []
-
-    def set_path(self, path):
-        self.path = path
-
-    def make_item(self):
-        item = SessionItem([self.name])
-
-        item.setData(0, Qt.UserRole, self.path)
-        if self.subfolders:
-            item.setIcon(0, QIcon.fromTheme('folder'))
-
-        if not self.path:
-            item.setFlags(item.flags() & ~Qt.ItemIsSelectable)
-
-        for folder in self.subfolders:
-            sub_item = folder.make_item()
-            item.addChild(sub_item)
-
-        return item
-
-class OpenSessionDialog(ChildDialog):
-    def __init__(self, parent):
-        ChildDialog.__init__(self, parent)
-        self.ui = ui.open_session.Ui_DialogOpenSession()
-        self.ui.setupUi(self)
-
-        self._timer_progress_n = 0
-        self._timer_progress = QTimer()
-        self._timer_progress.setInterval(50)
-        self._timer_progress.timeout.connect(self._timer_progress_timeout)
-        self._timer_progress.start()
-        self._progress_inverted = False
-
-        self.ui.widgetSpacer.setVisible(False)
-
-        self.ui.toolButtonFolder.clicked.connect(self._change_root_folder)
-        self.ui.sessionList.currentItemChanged.connect(
-            self._current_item_changed)
-        self.ui.sessionList.setFocus(Qt.OtherFocusReason)
-        self.ui.sessionList.itemDoubleClicked.connect(self._go_if_any)
-        self.ui.sessionList.itemClicked.connect(self._deploy_item)
-        self.ui.filterBar.textEdited.connect(self._update_filtered_list)
-        self.ui.filterBar.key_event.connect(self._up_down_pressed)
-        self.ui.buttonBox.button(QDialogButtonBox.Ok).setEnabled(False)
-        self.ui.currentSessionsFolder.setText(CommandLineArgs.session_root)
-
-        self.signaler.add_sessions_to_list.connect(self._add_sessions)
-        self.signaler.root_changed.connect(self._root_changed)
-
-        self.to_daemon('/ray/server/list_sessions', 0)
-
-        if not self.daemon_manager.is_local:
-            self.ui.toolButtonFolder.setVisible(False)
-            self.ui.currentSessionsFolder.setVisible(False)
-            self.ui.labelSessionsFolder.setVisible(False)
-
-        self._server_will_accept = False
-        self._has_selection = False
-        self._last_mouse_click = 0
-        self._last_session_item = None
-        
-        self._server_status_changed(self.session.server_status)
-
-        self.folders = []
-        self.all_items = []
-
-        self.ui.filterBar.setFocus(Qt.OtherFocusReason)
-
-    def _server_status_changed(self, server_status):
-        self.ui.toolButtonFolder.setEnabled(
-            bool(server_status in (ray.ServerStatus.OFF,
-                                   ray.ServerStatus.READY,
-                                   ray.ServerStatus.CLOSE)))
-
-        self._server_will_accept = bool(
-            server_status in (
-                ray.ServerStatus.OFF,
-                ray.ServerStatus.READY) and not self.server_copying)
-
-        if server_status != ray.ServerStatus.OFF:
-            if self._root_folder_file_dialog is not None:
-                self._root_folder_file_dialog.reject()
-            self._root_folder_message_box.reject()
-
-        self._prevent_ok()
-
-    def _timer_progress_timeout(self):
-        self.ui.progressBar.setValue(self._timer_progress_n)
-        if self._timer_progress_n >= 100:
-            self._timer_progress_n = 0
-            self._progress_inverted = not self._progress_inverted
-            self.ui.progressBar.setInvertedAppearance(
-                self._progress_inverted)
-        self._timer_progress_n += 5
-
-    def _root_changed(self, session_root):
-        self.ui.currentSessionsFolder.setText(session_root)
-        self.ui.sessionList.clear()
-        self.folders.clear()
-        self.to_daemon('/ray/server/list_sessions', 0)
-
-    def _add_sessions(self, session_names):
-        if not session_names:
-            self._timer_progress.stop()
-            height = self.ui.progressBar.size().height()
-            self.ui.progressBar.setVisible(False)
-            self.ui.widgetSpacer.setVisible(True)
-
-            # Try to select last used session
-            root_item = self.ui.sessionList.invisibleRootItem()
-            for i in range(root_item.childCount()):
-                item = root_item.child(i)
-                last_session_item = item.find_item_with(
-                    RS.settings.value('last_session', type=str))
-
-                if last_session_item:
-                    self.ui.sessionList.setCurrentItem(last_session_item)
-                    self.ui.sessionList.scrollToItem(last_session_item)
-                    break
-            return
-
-        for session_name in session_names:
-            folder_div = session_name.split('/')
-            folders = self.folders
-
-            for i in range(len(folder_div)):
-                f = folder_div[i]
-                for g in folders:
-                    if g.name == f:
-                        if i+1 == len(folder_div):
-                            g.set_path(session_name)
-
-                        folders = g.subfolders
-                        break
-                else:
-                    new_folder = SessionFolder(f)
-                    if i+1 == len(folder_div):
-                        new_folder.set_path(session_name)
-                    folders.append(new_folder)
-                    folders = new_folder.subfolders
-
-        self.ui.sessionList.clear()
-
-        for folder in self.folders:
-            item = folder.make_item()
-            self.ui.sessionList.addTopLevelItem(item)
-
-        self.ui.sessionList.sortByColumn(0, Qt.AscendingOrder)
-
-    def _update_filtered_list(self, filt):
-        filter_text = self.ui.filterBar.displayText()
-        root_item = self.ui.sessionList.invisibleRootItem()
-
-        ## hide all non matching items
-        for i in range(root_item.childCount()):
-            root_item.child(i).show_conditionnaly(filter_text)
-
-        # if selected item not in list, then select the first visible
-        if (not self.ui.sessionList.currentItem()
-                or self.ui.sessionList.currentItem().isHidden()):
-            for i in range(root_item.childCount()):
-                item = root_item.child(i)
-                if not item.isHidden():
-                    self.ui.sessionList.setCurrentItem(item)
-                    break
-
-        if (not self.ui.sessionList.currentItem()
-                or self.ui.sessionList.currentItem().isHidden()):
-            self.ui.filterBar.setStyleSheet(
-                "QLineEdit { background-color: red}")
-            self.ui.sessionList.setCurrentItem(None)
-        else:
-            self.ui.filterBar.setStyleSheet("")
-            self.ui.sessionList.scrollTo(self.ui.sessionList.currentIndex())
-
-    def _up_down_pressed(self, event):
-        start_item = self.ui.sessionList.currentItem()
-        QTreeWidget.keyPressEvent(self.ui.sessionList, event)
-        if not start_item:
-            return
-
-        current_item = self.ui.sessionList.currentItem()
-        if current_item == start_item:
-            return
-
-        ex_item = current_item
-
-        while not current_item.flags() & Qt.ItemIsSelectable:
-            ex_item = current_item
-            QTreeWidget.keyPressEvent(self.ui.sessionList, event)
-            current_item = self.ui.sessionList.currentItem()
-            if current_item == ex_item:
-                self.ui.sessionList.setCurrentItem(start_item)
-                return
-
-    def _current_item_changed(self, item, previous_item):
-        self._has_selection = bool(item and item.data(0, Qt.UserRole))
-        self._prevent_ok()
-
-    def _prevent_ok(self):
-        self.ui.buttonBox.button(QDialogButtonBox.Ok).setEnabled(
-            bool(self._server_will_accept and self._has_selection))
-
-    def _deploy_item(self, item, column):
-        if not item.childCount():
-            return
-
-        if time.time() - self._last_mouse_click > 0.35:
-            item.setExpanded(not item.isExpanded())
-
-        self._last_mouse_click = time.time()
-
-    def _go_if_any(self, item, column):
-        if item.childCount():
-            return
-
-        if (self._server_will_accept and self._has_selection
-                and self.ui.sessionList.currentItem().data(0, Qt.UserRole)):
-            self.accept()
-
-    def get_selected_session(self)->str:
-        if self.ui.sessionList.currentItem():
-            return self.ui.sessionList.currentItem().data(0, Qt.UserRole)
-
-
 class NewSessionDialog(ChildDialog):
     def __init__(self, parent, duplicate_window=False):
         ChildDialog.__init__(self, parent)
@@ -435,10 +164,12 @@ class NewSessionDialog(ChildDialog):
         if self._is_duplicate:
             self.ui.labelTemplate.setVisible(False)
             self.ui.comboBoxTemplate.setVisible(False)
+            self.ui.labelOriginalSessionName.setText(self.session.get_short_path())
             self.ui.labelNewSessionName.setText(
                 _translate('Duplicate', 'Duplicated session name :'))
             self.setWindowTitle(_translate('Duplicate', 'Duplicate Session'))
         else:
+            self.ui.frameOriginalSession.setVisible(False)
             self.to_daemon('/ray/server/list_session_templates')
 
         if not self.daemon_manager.is_local:
@@ -456,6 +187,8 @@ class NewSessionDialog(ChildDialog):
         self.ui.lineEdit.setCompleter(self._completer)
 
         self._server_status_changed(self.session.server_status)
+        
+        self._text_was_empty = True
 
     def _server_status_changed(self, server_status):
         self.ui.toolButtonFolder.setEnabled(
@@ -493,14 +226,15 @@ class NewSessionDialog(ChildDialog):
         self.ui.comboBoxTemplate.addItem(
             _translate('session_template', "with basic scripts"))
 
-        misscount = self.ui.comboBoxTemplate.count()  - 1 - len(
-                                                ray.factory_session_templates)
+        misscount = self.ui.comboBoxTemplate.count() \
+                    - 1 - len(ray.FACTORY_SESSION_TEMPLATES)
+
         for i in range(misscount):
             self.ui.comboBoxTemplate.addItem(
-                ray.factory_session_templates[-i])
+                ray.FACTORY_SESSION_TEMPLATES[-i])
 
         self.ui.comboBoxTemplate.insertSeparator(
-                                    len(ray.factory_session_templates) + 1)
+                                    len(ray.FACTORY_SESSION_TEMPLATES) + 1)
 
     def _set_last_template_selected(self):
         last_used_template = RS.settings.value('last_used_template', type=str)
@@ -508,8 +242,8 @@ class NewSessionDialog(ChildDialog):
         if last_used_template.startswith('///'):
             last_factory_template = last_used_template.replace('///', '', 1)
 
-            for i in range(len(ray.factory_session_templates)):
-                factory_template = ray.factory_session_templates[i]
+            for i in range(len(ray.FACTORY_SESSION_TEMPLATES)):
+                factory_template = ray.FACTORY_SESSION_TEMPLATES[i]
                 if factory_template == last_factory_template:
                     self.ui.comboBoxTemplate.setCurrentIndex(i+1)
                     break
@@ -521,7 +255,16 @@ class NewSessionDialog(ChildDialog):
             self.ui.comboBoxTemplate.setCurrentIndex(1)
 
     def _set_last_sub_folder_selected(self):
-        last_subfolder = RS.settings.value('last_subfolder', type=str)
+        last_subfolder = ''
+
+        for sess in self.session.recent_sessions:
+            if sess.startswith('/'):
+                continue
+
+            if '/' in sess:
+                last_subfolder, sep, sess_name = sess.rpartition('/')
+            break
+
         if last_subfolder and not self.ui.lineEdit.text():
             self.ui.lineEdit.setText(last_subfolder + '/')
 
@@ -539,7 +282,9 @@ class NewSessionDialog(ChildDialog):
         self._completer = QCompleter([f + '/' for f in self.sub_folders])
         self.ui.lineEdit.setCompleter(self._completer)
 
-        self._set_last_sub_folder_selected()
+        if not session_names:
+            # all sessions are listed, pre-fill last subfolder
+            self._set_last_sub_folder_selected()
 
     def _add_templates_to_list(self, template_list):
         for template in template_list:
@@ -564,6 +309,31 @@ class NewSessionDialog(ChildDialog):
                                    and text not in self.session_list)
         self._prevent_ok()
 
+        if self._text_was_empty:
+            if text:
+                self._completer.setCompletionMode(QCompleter.PopupCompletion)
+                self._completer.complete()
+                self._text_was_empty = False
+        
+        elif not text:
+            QTimer.singleShot(50, self._set_completer_for_empty_text)
+            self._text_was_empty = True
+
+    def _set_completer_for_empty_text(self):
+        #print('set currow', self._completer.setCurrentIndex(0))
+        del self._completer
+        self._completer = QCompleter([f + '/' for f in self.sub_folders])
+        self._completer.setCompletionMode(QCompleter.UnfilteredPopupCompletion)
+        self.ui.lineEdit.setCompleter(self._completer)
+        QTimer.singleShot(50, self._completer.complete)
+
+    def _show_completer_at_start(self):
+        self._completer.complete()
+
+    def showEvent(self, event):
+        ChildDialog.showEvent(self, event)
+        QTimer.singleShot(800, self._show_completer_at_start)
+
     def _prevent_ok(self):
         self.ui.buttonBox.button(QDialogButtonBox.Ok).setEnabled(
             bool(self._server_will_accept and self._text_is_valid))
@@ -577,8 +347,8 @@ class NewSessionDialog(ChildDialog):
         if index == 0:
             return ""
 
-        if index <= len(ray.factory_session_templates):
-            return '///' + ray.factory_session_templates[index-1]
+        if index <= len(ray.FACTORY_SESSION_TEMPLATES):
+            return '///' + ray.FACTORY_SESSION_TEMPLATES[index-1]
 
         return self.ui.comboBoxTemplate.currentText()
 
@@ -653,8 +423,7 @@ class SaveTemplateSessionDialog(AbstractSaveTemplateDialog):
     def __init__(self, parent):
         AbstractSaveTemplateDialog.__init__(self, parent)
         self.ui.toolButtonClientIcon.setVisible(False)
-        self.ui.labelLabel.setText(self.session.path)
-        self.template_list = []
+        self.ui.labelLabel.setText(self.session.get_short_path())
 
         self.signaler.session_template_found.connect(self._add_templates_to_list)
         self.to_daemon('/ray/server/list_session_templates')
@@ -676,10 +445,9 @@ class SaveTemplateClientDialog(AbstractSaveTemplateDialog):
         AbstractSaveTemplateDialog.__init__(self, parent)
         self.ui.labelSessionTitle.setVisible(False)
         self.ui.toolButtonClientIcon.setIcon(
-            ray.getAppIcon(client.icon, self))
+            get_app_icon(client.icon, self))
         self.ui.labelLabel.setText(client.prettier_name())
 
-        self.template_list = []
         self.ui.pushButtonAccept.setEnabled(False)
 
         self.ui.labelNewTemplateName.setText(
@@ -925,8 +693,8 @@ class QuitAppDialog(ChildDialog):
         self.ui.pushButtonQuitNoSave.clicked.connect(self._abort_session)
         self.ui.pushButtonDaemon.clicked.connect(self._leave_daemon_running)
 
-        original_text = self.ui.labelExecutable.text()
-        self.ui.labelExecutable.setText(
+        original_text = self.ui.labelMainText.text()
+        self.ui.labelMainText.setText(
             original_text %
             ('<strong>%s</strong>' %
              self.session.name))
@@ -959,6 +727,44 @@ class QuitAppDialog(ChildDialog):
         QTimer.singleShot(10, QGuiApplication.quit)
 
 
+class WrongVersionLocalDialog(ChildDialog):
+    def __init__(self, parent):
+        ChildDialog.__init__(self, parent)
+        self.ui = ui.quit_app.Ui_DialogQuitApp()
+        self.ui.setupUi(self)
+        self.ui.pushButtonCancel.setVisible(False)
+        self.ui.pushButtonSaveQuit.clicked.connect(self._close_session)
+        self.ui.pushButtonQuitNoSave.clicked.connect(self._abort_session)
+        self.ui.pushButtonDaemon.clicked.connect(self._leave_daemon_running)
+
+        #original_text = self.ui.labelMainText.text()
+        #self.ui.labelMainText.setText(
+            #original_text %
+            #('<strong>%s</strong>' %
+             #self.session.name))
+        self.ui.labelMainText.setText(
+            _translate(
+                'wrong_version',
+                "The running daemon has not the same version than the interface\n"
+                "RaySession will quit now.\n\n"
+                "What do you want to do with the current session ?"))
+
+    def _close_session(self):
+        # can make the GUI freeze a little
+        # but the GUI will quit just after
+        # and this case is rare enough to be acceptable
+        subprocess.run(['ray_control', 'close'])
+        self.accept()
+
+    def _abort_session(self):
+        # see _close_session
+        subprocess.run(['ray_control', 'abort'])
+        self.accept()
+
+    def _leave_daemon_running(self):
+        QTimer.singleShot(10, QGuiApplication.quit)
+
+
 class AboutRaySessionDialog(ChildDialog):
     def __init__(self, parent):
         ChildDialog.__init__(self, parent)
@@ -1049,14 +855,15 @@ class NewExecutableDialog(ChildDialog):
     def _close_now(self):
         if self._is_allowed():
             self.accept()
-    
+
     def get_selection(self)->tuple:
         return (self.ui.lineEdit.text(),
                 self.ui.checkBoxStartClient.isChecked(),
                 not self.ui.checkBoxNsm.isChecked(),
                 self.ui.comboBoxPrefixMode.currentIndex(),
                 self.ui.lineEditPrefix.text(),
-                self.ui.lineEditClientId.text())
+                self.ui.lineEditClientId.text(),
+                self.ui.checkBoxJackNaming.isChecked())
 
 
 class StopClientDialog(ChildDialog):
@@ -1138,7 +945,7 @@ class ClientRenameDialog(ChildDialog):
         self.ui.setupUi(self)
 
         self.client = client
-        self.ui.toolButtonIcon.setIcon(ray.getAppIcon(client.icon, self))
+        self.ui.toolButtonIcon.setIcon(get_app_icon(client.icon, self))
         self.ui.labelClientLabel.setText(client.prettier_name())
         self.ui.lineEdit.setText(client.prettier_name())
         self.ui.lineEdit.selectAll()
@@ -1288,7 +1095,7 @@ class DaemonUrlWindow(ChildDialog):
         self.ui.labelError.setText(error_text)
         self.ui.buttonBox.button(QDialogButtonBox.Ok).setEnabled(False)
 
-        self.tried_urls = ray.getListInSettings(RS.settings, 'network/tried_urls')
+        self.tried_urls = ray.get_list_in_settings(RS.settings, 'network/tried_urls')
         last_tried_url = RS.settings.value('network/last_tried_url', '', type=str)
 
         self._completer = QCompleter(self.tried_urls)
@@ -1310,7 +1117,7 @@ class DaemonUrlWindow(ChildDialog):
             return
 
         try:
-            addr = ray.getLibloAddress(text)
+            addr = ray.get_liblo_address(text)
             self.ui.buttonBox.button(QDialogButtonBox.Ok).setEnabled(True)
         except BaseException:
             self.ui.buttonBox.button(QDialogButtonBox.Ok).setEnabled(False)
@@ -1355,7 +1162,7 @@ class CanvasPortInfoDialog(ChildDialog):
         self.ui = ui.canvas_port_info.Ui_Dialog()
         self.ui.setupUi(self)
 
-    def set_infos(self, port_full_name: str, port_uuid: int, 
+    def set_infos(self, port_full_name: str, port_uuid: int,
                   port_type: str, port_flags: str,
                   pretty_name: str, port_order: int,
                   portgroup_name: str):
@@ -1369,7 +1176,7 @@ class CanvasPortInfoDialog(ChildDialog):
 
         if not (pretty_name or port_order or portgroup_name):
             self.ui.groupBoxMetadatas.setVisible(False)
-        
+
 
 class DonationsDialog(ChildDialog):
     def __init__(self, parent, display_no_again):
@@ -1384,6 +1191,141 @@ class DonationsDialog(ChildDialog):
         RS.set_hidden(RS.HD_Donations, state)
 
 
+class SystrayCloseDialog(ChildDialog):
+    def __init__(self, parent):
+        ChildDialog.__init__(self, parent)
+        self.ui = ui.systray_close.Ui_Dialog()
+        self.ui.setupUi(self)
+
+    def not_again(self)->bool:
+        return self.ui.checkBox.isChecked()
+
+
+class SystrayManagement(ChildDialog):
+    def __init__(self, parent):
+        ChildDialog.__init__(self, parent)
+        self.ui = ui.systray_management.Ui_Dialog()
+        self.ui.setupUi(self)
+
+        self.ui.groupBoxSystray.toggled.connect(
+            self.ui.checkBoxOnlySessionRunning.setEnabled)
+
+    def set_systray_mode(self, systray_mode: int):
+        self.ui.groupBoxSystray.setChecked(
+            systray_mode != ray.Systray.OFF)
+        self.ui.checkBoxOnlySessionRunning.setChecked(
+            systray_mode == ray.Systray.SESSION_ONLY)
+
+    def set_wild_shutdown(self, wild_shutdown: bool):
+        self.ui.checkBoxShutdown.setChecked(wild_shutdown)
+
+    def set_reversed_state(self, reversed: bool):
+        self.ui.checkBoxReversedMenu.setChecked(reversed)
+
+    def get_systray_mode(self)->int:
+        if self.ui.groupBoxSystray.isChecked():
+            if self.ui.checkBoxOnlySessionRunning.isChecked():
+                return ray.Systray.SESSION_ONLY
+            return ray.Systray.ALWAYS
+        return ray.Systray.OFF
+
+    def wild_shutdown(self)->bool:
+        return self.ui.checkBoxShutdown.isChecked()
+
+    def menu_should_be_reversed(self)->bool:
+        return self.ui.checkBoxReversedMenu.isChecked()
+
+
+class StartupDialog(ChildDialog):
+    ACTION_NO = 0
+    ACTION_NEW = 1
+    ACTION_OPEN = 2
+
+    def __init__(self, parent):
+        ChildDialog.__init__(self, parent)
+        self.ui = ui.startup_dialog.Ui_Dialog()
+        self.ui.setupUi(self)
+
+        self._clicked_action = self.ACTION_NO
+
+        self.ui.listWidgetRecentSessions.itemDoubleClicked.connect(
+            self.accept)
+
+        for recent_session in self.session.recent_sessions:
+            session_item = QListWidgetItem(recent_session.replace('/', ' / '),
+                                           self.ui.listWidgetRecentSessions)
+            session_item.setData(Qt.UserRole, recent_session)
+            self.ui.listWidgetRecentSessions.addItem(session_item)
+
+        self.ui.listWidgetRecentSessions.setMinimumHeight(
+            30 * len(self.session.recent_sessions))
+        self.ui.listWidgetRecentSessions.setCurrentRow(0)
+        self.ui.pushButtonNewSession.clicked.connect(
+            self._new_session_clicked)
+        self.ui.pushButtonOpenSession.clicked.connect(
+            self._open_session_clicked)
+        #self.ui.buttonBox.key_event.connect(self._up_down_pressed)
+        self.ui.pushButtonNewSession.focus_on_list.connect(
+            self._focus_on_list)
+        self.ui.pushButtonOpenSession.focus_on_list.connect(
+            self._focus_on_list)
+        self.ui.pushButtonNewSession.focus_on_open.connect(
+            self._focus_on_open)
+        self.ui.pushButtonOpenSession.focus_on_new.connect(
+            self._focus_on_new)
+
+        self.ui.listWidgetRecentSessions.setFocus(Qt.OtherFocusReason)
+
+    def _server_status_changed(self, server_status):
+        if server_status != ray.ServerStatus.OFF:
+            self.reject()
+
+    def _new_session_clicked(self):
+        self._clicked_action = self.ACTION_NEW
+        self.reject()
+
+    def _open_session_clicked(self):
+        self._clicked_action = self.ACTION_OPEN
+        self.reject()
+
+    def _focus_on_list(self):
+        self.ui.listWidgetRecentSessions.setFocus(Qt.OtherFocusReason)
+
+    def _focus_on_new(self):
+        self.ui.pushButtonNewSession.setFocus(Qt.OtherFocusReason)
+
+    def _focus_on_open(self):
+        self.ui.pushButtonOpenSession.setFocus(Qt.OtherFocusReason)
+
+    def not_again_value(self)->bool:
+        return not self.ui.checkBox.isChecked()
+
+    def get_selected_session(self)->str:
+        current_item = self.ui.listWidgetRecentSessions.currentItem()
+        if current_item:
+            return current_item.data(Qt.UserRole)
+        return ''
+
+    def get_clicked_action(self)->int:
+        return self._clicked_action
+
+    def keyPressEvent(self, event):
+        if event.key() == Qt.Key_Left:
+            self.ui.pushButtonNewSession.setFocus(Qt.OtherFocusReason)
+        elif event.key() == Qt.Key_Right:
+            self.ui.pushButtonOpenSession.setFocus(Qt.OtherFocusReason)
+        elif event.key() in (Qt.Key_Up, Qt.Key_Down):
+            self.ui.listWidgetRecentSessions.setFocus(Qt.OtherFocusReason)
+
+        if QApplication.keyboardModifiers() & Qt.ControlModifier:
+            if event.key() == Qt.Key_N:
+                self._new_session_clicked()
+            elif event.key() == Qt.Key_O:
+                self._open_session_clicked()
+
+        ChildDialog.keyPressEvent(self, event)
+
+
 class ErrorDialog(ChildDialog):
     def __init__(self, parent, message):
         ChildDialog.__init__(self, parent)
