@@ -49,7 +49,7 @@ class DaemonManager(QObject):
 
     def _change_url(self, new_url: str):
         try:
-            self.set_osc_address(ray.getLibloAddress(new_url))
+            self.set_osc_address(ray.get_liblo_address(new_url))
         except BaseException:
             return
 
@@ -90,7 +90,7 @@ class DaemonManager(QObject):
             options, session_root, is_net_free):
         self._announce_timer.stop()
 
-        if version.split('.')[:1] != ray.VERSION.split('.')[:1]:
+        if version.split('.')[:2] != ray.VERSION.split('.')[:2]:
             # works only if the two firsts digits are the same (ex: 0.6)
             self.signaler.daemon_url_request.emit(
                 ErrDaemon.WRONG_VERSION, self.url)
@@ -245,8 +245,11 @@ class DaemonManager(QObject):
 
     def set_new_osc_address(self):
         if not (self.address or self._port):
-            self._port = ray.getFreeOscPort()
+            self._port = ray.get_free_osc_port()
             self.address = Address(self._port)
 
     def is_announced(self):
         return self._is_announced
+
+    def get_port(self):
+        return self._port
