@@ -10,7 +10,7 @@ from daemon_manager import DaemonManager
 from gui_client import Client, TrashedClient
 from gui_signaler import Signaler
 from gui_server_thread import GuiServerThread
-from gui_tools import CommandLineArgs, RS
+from gui_tools import CommandLineArgs, RS, error_text
 from main_window import MainWindow
 from nsm_child import NsmChild, NsmChildOutside
 from patchbay_manager import PatchbayManager
@@ -21,6 +21,7 @@ class Session:
         self.client_list = []
         self.trashed_clients = []
         self.favorite_list = []
+        self.recent_sessions = []
         self.name = ''
         self.path = ''
         self.notes = ''
@@ -138,6 +139,12 @@ class SignaledSession(Session):
         self.canvas_groups = []
         self.canvas_ports = []
         self.next_canvas_port_id = -1
+        
+        self.preview_notes = ''
+        self.preview_client_list = []
+        self.preview_started_clients = set()
+        self.preview_snapshots = []
+        self.preview_size = -1
 
     def _osc_receive(self, path, args):
         func_path = path
@@ -168,6 +175,10 @@ class SignaledSession(Session):
                         ray.Err.COPY_ABORTED):
             return
 
+        message = error_text(err_code)
+        if message:
+            err_message = message
+
         self.main_win.error_message(err_message)
 
     def _minor_error(self, path, args):
@@ -196,6 +207,10 @@ class SignaledSession(Session):
         options = args[0]
         self.set_daemon_options(options)
 
+    def _ray_gui_server_recent_sessions(self, path, args):
+        self.recent_sessions = args
+        self.main_win.update_recent_sessions_menu()
+
     def _ray_gui_session_name(self, path, args):
         sname, spath = args
         self._set_name(sname)
@@ -316,6 +331,9 @@ class SignaledSession(Session):
         client = self.get_client(client_id)
         if client:
             client.set_gui_state(bool(int_state))
+            
+        self.patchbay_manager.optional_gui_state_changed(
+            client_id, bool(int_state))
 
     def _ray_gui_client_still_running(self, path, args):
         client_id = args[0]
@@ -341,14 +359,14 @@ class SignaledSession(Session):
         client_id = args.pop(0)
         for trashed_client in self.trashed_clients:
             if trashed_client.client_id == client_id:
-                trashed_client.ray_hack = ray.RayHack.newFrom(*args)
+                trashed_client.ray_hack = ray.RayHack.new_from(*args)
                 break
 
     def _ray_gui_trash_ray_net_update(self, path, args):
         client_id = args.pop(0)
         for trashed_client in self.trashed_clients:
             if trashed_client.client_id == client_id:
-                trashed_client.ray_net = ray.RayNet.newFrom(*args)
+                trashed_client.ray_net = ray.RayNet.new_from(*args)
                 break
 
     def _ray_gui_trash_remove(self, path, args):
@@ -397,6 +415,54 @@ class SignaledSession(Session):
         self.signaler.favorite_removed.emit(template_name, bool(int_factory))
         self.main_win.update_favorites_menu()
 
+    def _ray_gui_preview_clear(self, path, args):
+        self.preview_notes = ''
+        self.preview_client_list.clear()
+        self.preview_started_clients.clear()
+        self.preview_snapshots.clear()
+        self.preview_size = -1
+
+    def _ray_gui_preview_notes(self, path, args):
+        self.preview_notes = args[0]
+    
+    def _ray_gui_preview_client_update(self, path, args):
+        client = ray.ClientData.new_from(*args)
+        for pv_client in self.preview_client_list:
+            if pv_client.client_id == client.client_id:
+                pv_client.update(*args)
+                break
+        else:
+            self.preview_client_list.append(client)
+
+    def _ray_gui_preview_client_ray_hack_update(self, path, args):
+        client_id = args.pop(0)
+        for pv_client in self.preview_client_list:
+            if pv_client.client_id == client_id:
+                pv_client.set_ray_hack(ray.RayHack.new_from(*args))
+                break
+    
+    def _ray_gui_preview_client_ray_net_update(self, path, args):
+        client_id = args.pop(0)
+        for pv_client in self.preview_client_list:
+            if pv_client.client_id == client_id:
+                pv_client.set_ray_net(ray.RayNet.new_from(*args))
+                break
+
+    def _ray_gui_preview_client_is_started(self, path, args):
+        client_id, is_started = args
+
+        for pv_client in self.preview_client_list:
+            if pv_client.client_id == client_id:
+                if is_started:
+                    self.preview_started_clients.add(client_id)
+                break
+
+    def _ray_gui_preview_snapshot(self, path, args):
+        self.preview_snapshots.append(args[0])
+
+    def _ray_gui_preview_session_size(self, path, args):
+        self.preview_size = args[0]
+
     def _ray_gui_script_info(self, path, args):
         text = args[0]
         self.main_win.show_script_info(text)
