@@ -29,6 +29,16 @@ class Signaler(QObject):
 
     root_changed = pyqtSignal(str)
 
+    session_preview_update = pyqtSignal()
+    session_details = pyqtSignal(str, int, int, int)
+    scripted_dir = pyqtSignal(str, int)
+    parrallel_copy_state = pyqtSignal(int, int)
+    parrallel_copy_progress = pyqtSignal(int, float)
+    parrallel_copy_aborted = pyqtSignal()
+    other_session_renamed = pyqtSignal()
+    other_session_duplicated = pyqtSignal()
+    other_session_templated = pyqtSignal()
+
     client_added_reply = pyqtSignal(str)
 
     client_properties_state_changed = pyqtSignal(str, bool)
@@ -37,6 +47,7 @@ class Signaler(QObject):
     favorite_removed = pyqtSignal(str, bool)
 
     canvas_callback = pyqtSignal(int, int, int, str)
+    port_types_view_changed = pyqtSignal(int)
 
     def __init__(self):
         QObject.__init__(self)
