@@ -5,7 +5,7 @@ from PyQt5.QtCore import pyqtSlot, QSize
 import ray
 from gui_server_thread import GuiServerThread
 from gui_tools import (client_status_string, _translate, is_dark_theme,
-                       RayIcon, split_in_two)
+                       RayIcon, split_in_two, get_app_icon)
 import child_dialogs
 import snapshots_dialog
 
@@ -48,6 +48,8 @@ class ClientSlot(QFrame):
         self.ui.actionRename.triggered.connect(self._rename_dialog)
         self.ui.actionReturnToAPreviousState.triggered.connect(
             self._open_snapshots_dialog)
+        self.ui.actionFindBoxesInPatchbay.triggered.connect(
+            self._find_patchbay_boxes)
         self.ui.actionProperties.triggered.connect(
             self.client.show_properties_dialog)
 
@@ -56,6 +58,7 @@ class ClientSlot(QFrame):
         self._menu.addAction(self.ui.actionSaveAsApplicationTemplate)
         self._menu.addAction(self.ui.actionRename)
         self._menu.addAction(self.ui.actionReturnToAPreviousState)
+        self._menu.addAction(self.ui.actionFindBoxesInPatchbay)
         self._menu.addAction(self.ui.actionProperties)
 
         self.ui.actionReturnToAPreviousState.setVisible(
@@ -150,6 +153,11 @@ class ClientSlot(QFrame):
             self.to_daemon('/ray/client/open_snapshot',
                           self.get_client_id(), snapshot)
 
+    def _find_patchbay_boxes(self):
+        self.main_win.set_patchbay_filter_text(
+            'client:' + self.get_client_id())
+        self._list_widget_item.setSelected(True)
+
     def _rename_dialog(self):
         dialog = child_dialogs.ClientRenameDialog(self.main_win,
                                                   self.client)
@@ -267,7 +275,7 @@ class ClientSlot(QFrame):
         tool_tip += "<p></p>"
         tool_tip += "<p>%s : %s<br>" \
             % (_translate('client_slot', 'Protocol'),
-               ray.protocolToStr(self.client.protocol))
+               ray.protocol_to_str(self.client.protocol))
         tool_tip += "%s : %s<br>" \
             % (_translate('client_slot', 'Executable'),
                self.client.executable_path)
@@ -278,7 +286,7 @@ class ClientSlot(QFrame):
         self.ui.ClientName.setToolTip(tool_tip)
 
         # set icon
-        self._icon_on = ray.getAppIcon(self.client.icon, self)
+        self._icon_on = get_app_icon(self.client.icon, self)
         self._icon_off = QIcon(self._icon_on.pixmap(32, 32, QIcon.Disabled))
 
         self._gray_icon(
@@ -318,7 +326,9 @@ class ClientSlot(QFrame):
         self.ui.lineEditClientStatus.setText(client_status_string(status))
         self.ui.lineEditClientStatus.setEnabled(
             status != ray.ClientStatus.STOPPED)
-        
+        self.ui.actionFindBoxesInPatchbay.setEnabled(
+            status not in (ray.ClientStatus.STOPPED, ray.ClientStatus.PRECOPY)) 
+
         ray_hack = bool(self.client.protocol == ray.Protocol.RAY_HACK)
 
         if status in (
@@ -436,6 +446,9 @@ class ClientSlot(QFrame):
         has_git = bool(options & ray.Option.HAS_GIT)
         self.ui.actionReturnToAPreviousState.setVisible(has_git)
 
+    def patchbay_is_shown(self, yesno: bool):
+        self.ui.actionFindBoxesInPatchbay.setVisible(yesno)
+
     def contextMenuEvent(self, event):
         act_selected = self._menu.exec(self.mapToGlobal(event.pos()))
         event.accept()
@@ -444,10 +457,10 @@ class ClientSlot(QFrame):
 class ClientItem(QListWidgetItem):
     def __init__(self, parent, client_data):
         QListWidgetItem.__init__(self, parent, QListWidgetItem.UserType + 1)
-        
+
         self.sort_number = 0
         self.widget = ClientSlot(parent, self, client_data)
-        
+
         parent.setItemWidget(self, self.widget)
         self.setSizeHint(QSize(100, 45))
 
@@ -505,6 +518,12 @@ class ListWidgetClients(QListWidget):
     def set_session(self, session):
         self.session = session
 
+    def patchbay_is_shown(self, yesno: bool):
+        for i in range(self.count()):
+            item = self.item(i)
+            widget = item.widget
+            widget.patchbay_is_shown(yesno)
+
     def dropEvent(self, event):
         QListWidget.dropEvent(self, event)
 
@@ -543,7 +562,7 @@ class ListWidgetClients(QListWidget):
 
                 for favorite in self.session.favorite_list:
                     act_app = fav_menu.addAction(
-                        ray.getAppIcon(favorite.icon, self), favorite.name)
+                        get_app_icon(favorite.icon, self), favorite.name)
                     act_app.setData([favorite.name, favorite.factory])
                     act_app.triggered.connect(self._launch_favorite)
 
