@@ -1,18 +1,24 @@
 import time
 import os
+import subprocess
+
 from PyQt5.QtWidgets import (QApplication, QMainWindow, QMenu, QDialog,
                              QMessageBox, QToolButton, QAbstractItemView,
-                             QBoxLayout)
-from PyQt5.QtGui import QIcon, QDesktopServices, QFontMetrics
+                             QBoxLayout, QSystemTrayIcon, QAction, QShortcut)
+from PyQt5.QtGui import QIcon, QDesktopServices, QFontMetrics, QKeySequence
 from PyQt5.QtCore import QTimer, pyqtSlot, QUrl, QLocale, Qt
 
-from gui_tools import (RS, RayIcon, CommandLineArgs, _translate,
-                       server_status_string, is_dark_theme, get_code_root)
+from gui_tools import (
+    RS, RayIcon, CommandLineArgs, _translate, server_status_string,
+    is_dark_theme, get_code_root, get_app_icon)
 import add_application_dialog
+import open_session_dialog
 import child_dialogs
 import snapshots_dialog
 from gui_server_thread import GuiServerThread
 from patchcanvas import patchcanvas
+import patchbay_manager
+from utility_scripts import UtilityScriptLauncher
 import ray
 import list_widget_clients
 
@@ -38,6 +44,8 @@ class MainWindow(QMainWindow):
 
         self.notes_dialog = None
 
+        self.util_script_launcher = UtilityScriptLauncher(self, session)
+
         # timer for keep focus while client opening
         self._timer_raisewin = QTimer()
         self._timer_raisewin.setInterval(50)
@@ -58,12 +66,12 @@ class MainWindow(QMainWindow):
 
         self.server_copying = False
 
-        self._keep_focus = RS.settings.value('keepfocus', True, type=bool)
+        self._keep_focus = RS.settings.value('keepfocus', False, type=bool)
         self.ui.actionKeepFocus.setChecked(self._keep_focus)
 
         # do not enable keep focus option under Wayland
         # because activate a window from it self on Wayland not allowed
-        if ray.getWindowManager() == ray.WindowManager.WAYLAND:
+        if ray.get_window_manager() == ray.WindowManager.WAYLAND:
             self._keep_focus = False
             self.ui.actionKeepFocus.setEnabled(False)
 
@@ -78,7 +86,7 @@ class MainWindow(QMainWindow):
 
         # manage geometry depending of use of embedded jack patchbay
         show_patchbay = RS.settings.value(
-            'MainWindow/show_patchbay', False, type=bool)
+            'MainWindow/show_patchbay', True, type=bool)
         self.ui.actionShowJackPatchbay.setChecked(show_patchbay)
         self.waiting_for_patchbay = show_patchbay
 
@@ -90,10 +98,10 @@ class MainWindow(QMainWindow):
             self.ui.graphicsView.setVisible(True)
 
             splitter_sizes = RS.settings.value(
-                'MainWindow/splitter_canvas_sizes')
-            if splitter_sizes:
-                self.ui.splitterMainVsCanvas.setSizes(
-                    int(s) for s in splitter_sizes)
+                'MainWindow/splitter_canvas_sizes', [420, 706], type=list)
+
+            self.ui.splitterMainVsCanvas.setSizes(
+                int(s) for s in splitter_sizes)
 
         else:
             self.ui.graphicsView.setVisible(False)
@@ -112,10 +120,11 @@ class MainWindow(QMainWindow):
                 self.setMinimumWidth(450)
                 self.setGeometry(x, y, 460, height)
 
-        splitter_sizes = RS.settings.value("MainWindow/splitter_messages")
-        if splitter_sizes:
-            self.ui.splitterSessionVsMessages.setSizes(
-                [int(s) for s in splitter_sizes])
+        splitter_sizes = RS.settings.value(
+            "MainWindow/splitter_messages", [466, 84], type=list)
+
+        self.ui.splitterSessionVsMessages.setSizes(
+            [int(s) for s in splitter_sizes])
 
         if RS.settings.value('MainWindow/WindowState'):
             self.restoreState(RS.settings.value('MainWindow/WindowState'))
@@ -147,6 +156,14 @@ class MainWindow(QMainWindow):
         # connect actions
         self.ui.actionNewSession.triggered.connect(self._create_new_session)
         self.ui.actionOpenSession.triggered.connect(self._open_session)
+        self.ui.actionConvertArdourSession.triggered.connect(
+            self.util_script_launcher.convert_ardour_to_session)
+        self.ui.actionConvertHydrogenRhNsm.triggered.connect(
+            self.util_script_launcher.convert_ray_hack_to_nsm_hydrogen)
+        self.ui.actionConvertJackMixerRhNsm.triggered.connect(
+            self.util_script_launcher.convert_ray_hack_to_nsm_jack_mixer)
+        self.ui.actionConvertToNsmFileFormat.triggered.connect(
+            self.util_script_launcher.convert_to_nsm_file_format)
         self.ui.actionQuit.triggered.connect(self._quit_app)
         self.ui.actionSaveSession.triggered.connect(self._save_session)
         self.ui.actionCloseSession.triggered.connect(self._close_session)
@@ -188,6 +205,8 @@ class MainWindow(QMainWindow):
         self.ui.actionOnlineManual.triggered.connect(self._online_manual)
         self.ui.actionInternalManual.triggered.connect(self._internal_manual)
         self.ui.actionDonate.triggered.connect(self.donate)
+        self.ui.actionSystemTrayIconOptions.triggered.connect(
+            self._open_systray_options)
         self.ui.actionMakeReappearDialogs.triggered.connect(
             self._make_all_dialogs_reappear)
 
@@ -285,6 +304,8 @@ class MainWindow(QMainWindow):
         self.ui.actionRememberOptionalGuiStates.setIcon(
             RayIcon('visibility', dark))
         self.ui.trashButton.setIcon(RayIcon('trash-empty', dark))
+        if self.ui.trashButton.icon().isNull():
+            self.ui.trashButton.setIcon(RayIcon('trash'), dark)
 
         self.ui.actionDuplicateSession.setIcon(
             RayIcon('xml-node-duplicate', dark))
@@ -305,8 +326,16 @@ class MainWindow(QMainWindow):
         self.ui.toolButtonSessionMenu.setIcon(RayIcon('application-menu', dark))
 
         self.ui.listWidget.set_session(self.session)
+        
+        # concerns patchbay filters bar (activable with Ctrl+F)
+        self.ui.framePatchbayFilters.set_patchbay_manager(
+            self.session.patchbay_manager)
+        self.ui.framePatchbayFilters.setVisible(False)
+        filter_bar_shortcut = QShortcut('Ctrl+F', self)
+        filter_bar_shortcut.setContext(Qt.ApplicationShortcut)
+        filter_bar_shortcut.activated.connect(self.toggle_patchbay_filters_bar)
 
-        # prevent to hide the session frame with splitter
+        # prevent to hide the session frame with session/messages splitter
         self.ui.splitterSessionVsMessages.setCollapsible(0, False)
         self.ui.splitterSessionVsMessages.splitterMoved.connect(
             self._splitter_session_vs_messages_moved)
@@ -338,6 +367,32 @@ class MainWindow(QMainWindow):
         self._geom_before_fullscreen = None
         self._splitter_pos_before_fullscreen = [100, 100]
         self._fullscreen_patchbay = False
+        self.hidden_maximized = False
+
+        # systray icon and related
+        self._wild_shutdown = RS.settings.value(
+            'wild_shutdown', False, type=bool)
+        self._systray_mode = RS.settings.value(
+            'systray_mode', ray.Systray.SESSION_ONLY, type=int)
+        self._reversed_systray_menu = RS.settings.value(
+            'reversed_systray_menu', False, type=bool)
+
+        self._systray = QSystemTrayIcon(self)
+        self._systray.activated.connect(self._systray_activated)
+        self._systray.setIcon(QIcon(':48x48/raysession'))
+        self._systray.setToolTip(ray.APP_TITLE)
+        self._systray_menu = QMenu()
+        self._systray_menu_add = QMenu(self._systray_menu)
+
+        self._build_systray_menu()
+
+        if (not CommandLineArgs.under_nsm
+            and (self._systray_mode == ray.Systray.ALWAYS
+                    or (self._systray_mode == ray.Systray.SESSION_ONLY
+                        and self.session.server_status != ray.ServerStatus.OFF))):
+            self._systray.show()
+
+        self._startup_time = time.time()
 
     def _splitter_session_vs_messages_moved(self, pos: int, index: int):
         self.ui.actionToggleShowMessages.setChecked(
@@ -368,7 +423,7 @@ class MainWindow(QMainWindow):
             self.ui.widgetPreRewindSpacer.setVisible(True)
         else:
             #self.ui.toolBar.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
-            
+
             self.ui.layoutSessionDown.setDirection(QBoxLayout.LeftToRight)
             self.ui.layoutSessionDown.removeWidget(
                 self.ui.stackedWidgetSessionName)
@@ -421,11 +476,15 @@ class MainWindow(QMainWindow):
         if RS.settings.value('Canvas/box_shadows', False, type=bool):
             options.eyecandy = patchcanvas.EYECANDY_SMALL
 
-        options.auto_hide_groups = False
+        options.auto_hide_groups = True
         options.auto_select_items = False
         options.inline_displays = False
         options.use_bezier_lines = True
         options.elastic = RS.settings.value('Canvas/elastic', True, type=bool)
+        options.prevent_overlap = RS.settings.value(
+            'Canvas/prevent_overlap', True, type=bool)
+        options.max_port_width = RS.settings.value(
+            'Canvas/max_port_width', 160, type=int)
 
         features = patchcanvas.features_t()
         features.group_info = False
@@ -439,10 +498,43 @@ class MainWindow(QMainWindow):
         patchcanvas.init(
             ray.APP_TITLE, self.scene,
             self.canvas_callback, False)
+        patchcanvas.set_semi_hide_opacity(RS.settings.value(
+            'Canvas/semi_hide_opacity', 0.17, type=float))
 
     def _open_file_manager(self):
         self.to_daemon('/ray/session/open_folder')
 
+    def _open_systray_options(self):
+        dialog = child_dialogs.SystrayManagement(self)
+        dialog.set_systray_mode(self._systray_mode)
+        dialog.set_wild_shutdown(self._wild_shutdown)
+        dialog.set_reversed_state(self._reversed_systray_menu)
+
+        dialog.exec()
+        if not dialog.result():
+            return
+
+        self._systray_mode = dialog.get_systray_mode()
+        self._wild_shutdown = dialog.wild_shutdown()
+        reversed_systray_menu = dialog.menu_should_be_reversed()
+        RS.settings.setValue('systray_mode', self._systray_mode)
+        RS.settings.setValue('wild_shutdown', self._wild_shutdown)
+        RS.settings.setValue('reversed_systray_menu', reversed_systray_menu)
+
+        if reversed_systray_menu != self._reversed_systray_menu:
+            self._reversed_systray_menu = reversed_systray_menu
+            self._build_systray_menu()
+
+        if self._systray_mode == ray.Systray.OFF:
+            self._systray.hide()
+        elif self._systray_mode == ray.Systray.SESSION_ONLY:
+            if self.session.server_status == ray.ServerStatus.OFF:
+                self._systray.hide()
+            else:
+                self._systray.show()
+        elif self._systray_mode == ray.Systray.ALWAYS:
+            self._systray.show()
+
     def _raise_window(self):
         if self.mouse_is_inside:
             self.activateWindow()
@@ -482,7 +574,13 @@ class MainWindow(QMainWindow):
         self._flash_open_bool = not self._flash_open_bool
 
     def _quit_app(self):
+        if self._wild_shutdown and not CommandLineArgs.under_nsm:
+            self.daemon_manager.disannounce()
+            QTimer.singleShot(10, QApplication.quit)
+            return
+
         if self.session.is_running():
+            self.show()
             dialog = child_dialogs.QuitAppDialog(self)
             dialog.exec()
             if not dialog.result():
@@ -495,6 +593,10 @@ class MainWindow(QMainWindow):
         self.daemon_manager.stop()
 
     def _create_new_session(self):
+        # from systray menu, better to show main window in the background
+        # before open dialog
+        self.show()
+
         dialog = child_dialogs.NewSessionDialog(self)
         dialog.exec()
         if not dialog.result():
@@ -505,19 +607,13 @@ class MainWindow(QMainWindow):
         subfolder = session_short_path.rpartition('/')[0]
 
         RS.settings.setValue('last_used_template', template_name)
-        RS.settings.setValue('last_subfolder', subfolder)
-        if self.session.is_running():
-            # remember the running session as last session (if any)
-            short_path = self.session.get_short_path()
-            if not short_path.startswith('/'):
-                RS.settings.setValue('last_session', short_path)
 
         if not template_name:
             self.to_daemon('/ray/server/new_session', session_short_path)
             return
 
         if template_name.startswith('///'):
-            if template_name == '///' + ray.factory_session_templates[1]:
+            if template_name == '///' + ray.FACTORY_SESSION_TEMPLATES[1]:
                 if not RS.is_hidden(RS.HD_JackConfigScript):
                     # display jack_config_script info dialog
                     # and manage ray-jack_checker auto_start
@@ -540,7 +636,7 @@ class MainWindow(QMainWindow):
 
                     self.to_daemon('/ray/server/exotic_action', action)
 
-            elif template_name == '///' + ray.factory_session_templates[2]:
+            elif template_name == '///' + ray.FACTORY_SESSION_TEMPLATES[2]:
                 if not RS.is_hidden(RS.HD_SessionScripts):
                     # display session scripts info dialog
                     session_path = "%s/%s" % (CommandLineArgs.session_root,
@@ -556,28 +652,30 @@ class MainWindow(QMainWindow):
 
         self.to_daemon('/ray/server/new_session', session_short_path, template_name)
 
-    def _open_session(self, action):
-        dialog = child_dialogs.OpenSessionDialog(self)
+    def _open_session(self):
+        # from systray, better to show main window in the background
+        # before open dialog
+        self.show()
+
+        dialog = open_session_dialog.OpenSessionDialog(self)
         dialog.exec()
         if not dialog.result():
             return
 
-        if self.session.is_running():
-            RS.settings.setValue('last_session', self.session.get_short_path())
-
         session_name = dialog.get_selected_session()
-        self.to_daemon('/ray/server/open_session', session_name)
+        save_previous = int(dialog.want_to_save_previous())
+
+        self.to_daemon('/ray/server/open_session', session_name, save_previous)
 
     def _close_session(self):
-        RS.settings.setValue('last_session', self.session.get_short_path())
         self.to_daemon('/ray/session/close')
 
     def _abort_session(self):
+        self.show()
         dialog = child_dialogs.AbortSessionDialog(self)
         dialog.exec()
 
         if dialog.result():
-            RS.settings.setValue('last_session', self.session.get_short_path())
             self.to_daemon('/ray/session/abort')
 
     def _rename_session_action(self):
@@ -599,12 +697,6 @@ class MainWindow(QMainWindow):
         if not dialog.result():
             return
 
-        if self.session.is_running():
-            # remember the running session as last session (if any)
-            short_path = self.session.get_short_path()
-            if not short_path.startswith('/'):
-                RS.settings.setValue('last_session', short_path)
-
         session_name = dialog.get_session_short_path()
         self.to_daemon('/ray/session/duplicate', session_name)
 
@@ -691,10 +783,11 @@ class MainWindow(QMainWindow):
             return
 
         command, auto_start, via_proxy, \
-            prefix_mode, prefix, client_id = dialog.get_selection()
+            prefix_mode, prefix, client_id, jack_naming = dialog.get_selection()
 
-        self.to_daemon('/ray/session/add_executable', command, int(auto_start),
-                       int(via_proxy), prefix_mode, prefix, client_id)
+        self.to_daemon(
+            '/ray/session/add_executable', command, int(auto_start),
+            int(via_proxy), prefix_mode, prefix, client_id, int(jack_naming))
 
     def _show_jack_patchbay(self, yesno: bool):
         self.save_window_settings(
@@ -738,11 +831,13 @@ class MainWindow(QMainWindow):
             self.ui.splitterMainVsCanvas.setSizes([100, 0])
 
         self.ui.graphicsView.setVisible(yesno)
+        self.ui.framePatchbayFilters.setVisible(False)
         self.ui.splitterMainVsCanvas.set_active(yesno)
+        self.ui.listWidget.patchbay_is_shown(yesno)
 
     def _status_bar_pressed(self):
         status = self.session.server_status
-        
+
         if status not in (
                 ray.ServerStatus.PRECOPY,
                 ray.ServerStatus.COPY,
@@ -791,6 +886,27 @@ class MainWindow(QMainWindow):
         self.to_daemon('/ray/server/abort_snapshot')
 
     def _show_daemon_url_window(self, err_code, ex_url=''):
+        if not CommandLineArgs.under_nsm:
+            server = GuiServerThread.instance()
+            if server and ray.are_on_same_machine(server.url, ex_url):
+                # here we are in the case daemon and GUI have not the same VERSION
+                # If a session is running, inform user
+                # else, just stop the daemon and quit
+                
+                session_path = subprocess.run(
+                    ['ray_control', 'get_session_path'], capture_output=True)
+                if session_path.stdout:
+                    dialog = child_dialogs.WrongVersionLocalDialog(self)
+                    dialog.exec()
+                    if dialog.result():
+                        subprocess.run(['ray_control', 'quit'])
+                        self._quit_app_now()
+                    
+                else:
+                    subprocess.run(['ray_control', 'quit'])
+                    self._quit_app_now()
+                return
+        
         dialog = child_dialogs.DaemonUrlWindow(self, err_code, ex_url)
         dialog.exec()
         if not dialog.result():
@@ -801,7 +917,7 @@ class MainWindow(QMainWindow):
 
         new_url = dialog.get_url()
 
-        tried_urls = ray.getListInSettings(RS.settings, 'network/tried_urls')
+        tried_urls = ray.get_list_in_settings(RS.settings, 'network/tried_urls')
         if new_url not in tried_urls:
             tried_urls.append(new_url)
 
@@ -830,6 +946,12 @@ class MainWindow(QMainWindow):
         self.ui.frameCurrentSession.setEnabled(
             bool(server_status != ray.ServerStatus.OFF))
 
+        if self._systray_mode == ray.Systray.SESSION_ONLY:
+            if server_status == ray.ServerStatus.OFF:
+                self._systray.hide()
+            else:
+                self._systray.show()
+
         if server_status in (ray.ServerStatus.SNAPSHOT,
                              ray.ServerStatus.OUT_SNAPSHOT):
             self._timer_snapshot.start()
@@ -896,6 +1018,8 @@ class MainWindow(QMainWindow):
         self.ui.trashButton.setEnabled(bool(self.session.trashed_clients)
                                        and not close_or_off)
 
+        self._systray_menu_add.setEnabled(not close_or_off)
+
         if (CommandLineArgs.under_nsm
                 and not CommandLineArgs.out_daemon
                 and ready
@@ -912,6 +1036,7 @@ class MainWindow(QMainWindow):
             self.ui.actionDuplicateSession.setEnabled(False)
             self.ui.actionCloseSession.setEnabled(False)
             self.ui.actionAbortSession.setEnabled(False)
+            self.ui.menuRecentSessions.setEnabled(False)
 
         if server_status == ray.ServerStatus.OFF:
             if self.terminate_request:
@@ -934,6 +1059,80 @@ class MainWindow(QMainWindow):
 
         RS.reset_hiddens()
 
+    def _build_systray_menu(self):
+        is_shown = self._systray.isVisible()
+        self._systray.hide()
+        self._systray_menu.clear()
+        self._systray_menu_add.clear()
+
+        del self._systray_menu
+        del self._systray_menu_add
+
+        self._systray_menu = QMenu()
+        self._systray_menu_add = QMenu(
+            _translate('menu', 'Add'), self._systray_menu)
+        self._systray_menu_add.addMenu(self._favorites_menu)
+        self._systray_menu_add.addAction(self.ui.actionAddApplication)
+        self._systray_menu_add.addAction(self.ui.actionAddExecutable)
+        self._systray_menu_add.setIcon(QIcon.fromTheme('list-add'))
+        self._systray_menu_add.setEnabled(
+            self.session.server_status not in
+                (ray.ServerStatus.OFF, ray.ServerStatus.CLOSE,
+                 ray.ServerStatus.WAIT_USER, ray.ServerStatus.OUT_SAVE,
+                 ray.ServerStatus.OUT_SNAPSHOT))
+
+        if self._reversed_systray_menu:
+            self._systray_menu.addAction(self.ui.actionQuit)
+            self._systray_menu.addSeparator()
+            self._systray_menu.addAction(self.ui.actionSystemTrayIconOptions)
+            self._systray_menu.addSeparator()
+            self._systray_menu.addMenu(self.ui.menuRecentSessions)
+            self._systray_menu.addAction(self.ui.actionOpenSession)
+            self._systray_menu.addAction(self.ui.actionNewSession)
+            self._systray_menu.addSeparator()
+            self._systray_menu.addAction(self.ui.actionAbortSession)
+            self._systray_menu.addAction(self.ui.actionCloseSession)
+            self._systray_menu.addMenu(self._systray_menu_add)
+            self._systray_menu.addAction(self.ui.actionSaveSession)
+        else:
+            self._systray_menu.addAction(self.ui.actionSaveSession)
+            self._systray_menu.addMenu(self._systray_menu_add)
+            self._systray_menu.addAction(self.ui.actionCloseSession)
+            self._systray_menu.addAction(self.ui.actionAbortSession)
+            self._systray_menu.addSeparator()
+            self._systray_menu.addAction(self.ui.actionNewSession)
+            self._systray_menu.addAction(self.ui.actionOpenSession)
+            self._systray_menu.addMenu(self.ui.menuRecentSessions)
+            self._systray_menu.addSeparator()
+            self._systray_menu.addAction(self.ui.actionSystemTrayIconOptions)
+            self._systray_menu.addSeparator()
+            self._systray_menu.addAction(self.ui.actionQuit)
+        self._systray.setContextMenu(self._systray_menu)
+
+        if is_shown:
+            self._systray.show()
+
+    def _systray_activated(self):
+        wayland = bool(ray.get_window_manager() == ray.WindowManager.WAYLAND)
+
+        if self.isMinimized():
+            if self.hidden_maximized:
+                self.showMaximized()
+            else:
+                self.showNormal()
+            if not wayland:
+                self.activateWindow()
+        elif self.isHidden():
+            self.show()
+            if not wayland:
+                self.activateWindow()
+        elif self.isActiveWindow():
+            self.hide()
+        elif wayland:
+            self.hide()
+        else:
+            self.activateWindow()
+
     ###FUNCTIONS RELATED TO SIGNALS FROM OSC SERVER#######
 
     def toggle_scene_full_screen(self):
@@ -955,7 +1154,7 @@ class MainWindow(QMainWindow):
 
             self.ui.splitterMainVsCanvas.setSizes(
                 self._splitter_pos_before_fullscreen)
-            
+
             self._fullscreen_patchbay = False
         else:
             self._were_visible_before_fullscreen = \
@@ -1026,9 +1225,9 @@ class MainWindow(QMainWindow):
             nsm_locked and not CommandLineArgs.out_daemon)
         self.ui.actionRenameSession_2.setEnabled(
             nsm_locked and not CommandLineArgs.out_daemon)
-        
+
         frame_style_sheet = "SessionFrame{border-radius:4px;"
-        
+
         if nsm_locked and CommandLineArgs.out_daemon:
             frame_style_sheet += "background-color: rgba(100, 181, 100, 35)}"
         elif nsm_locked:
@@ -1175,9 +1374,50 @@ class MainWindow(QMainWindow):
             if self.notes_dialog is not None:
                 self.notes_dialog.hide()
 
+        self.update_recent_sessions_menu()
+
     def set_session_name_editable(self, set_edit: bool):
         self.ui.stackedWidgetSessionName.set_editable(set_edit)
 
+    def update_recent_sessions_menu(self):
+        self.ui.menuRecentSessions.clear()
+
+        for sess in self.session.recent_sessions:
+            sess_action = self.ui.menuRecentSessions.addAction(sess)
+
+            if sess == self.session.get_short_path():
+                # disable running session
+                sess_action.setEnabled(False)
+
+            sess_action.setData(sess)
+            sess_action.triggered.connect(self.launch_recent_session)
+
+        self.ui.menuRecentSessions.setEnabled(bool(self.session.recent_sessions))
+        self._build_systray_menu()
+
+        # here we start the startup dialog
+        # FIXME - not a good place
+        if (not RS.is_hidden(RS.HD_StartupRecentSessions)
+                and time.time() - self._startup_time < 5
+                and self.session.recent_sessions
+                and self.session.server_status == ray.ServerStatus.OFF):
+            # ahah, dirty way to prevent a dialog once again
+            self._startup_time -= 5
+
+            dialog = child_dialogs.StartupDialog(self)
+            dialog.exec()
+
+            if dialog.result():
+                self.to_daemon('/ray/server/open_session',
+                               dialog.get_selected_session())
+            elif dialog.get_clicked_action() == dialog.ACTION_NEW:
+                self._create_new_session()
+            elif dialog.get_clicked_action() == dialog.ACTION_OPEN:
+                self._open_session()
+
+            if dialog.not_again_value():
+                RS.set_hidden(RS.HD_StartupRecentSessions)
+
     def error_message(self, message: str):
         error_dialog = child_dialogs.ErrorDialog(self, message)
         error_dialog.exec()
@@ -1191,7 +1431,7 @@ class MainWindow(QMainWindow):
 
     def trash_add(self, trashed_client):
         act_x_trashed = self._trash_menu.addAction(
-            ray.getAppIcon(trashed_client.icon, self),
+            get_app_icon(trashed_client.icon, self),
             trashed_client.prettier_name())
         act_x_trashed.setData(trashed_client.client_id)
         act_x_trashed.triggered.connect(self.show_client_trash_dialog)
@@ -1214,6 +1454,15 @@ class MainWindow(QMainWindow):
         self._trash_menu.clear()
         self.ui.trashButton.setEnabled(False)
 
+    @pyqtSlot()
+    def launch_recent_session(self):
+        try:
+            session_name = str(self.sender().data())
+        except BaseException:
+            return
+
+        self.to_daemon('/ray/server/open_session', session_name)
+
     @pyqtSlot()
     def show_client_trash_dialog(self):
         try:
@@ -1240,6 +1489,18 @@ class MainWindow(QMainWindow):
         self.to_daemon('/ray/session/add_client_template',
                        int(factory), template_name)
 
+    def toggle_patchbay_filters_bar(self):
+        if not self.ui.graphicsView.isVisible():
+            return
+
+        self.ui.framePatchbayFilters.setVisible(
+            not self.ui.framePatchbayFilters.isVisible())
+
+    def set_patchbay_filter_text(self, text: str):
+        ''' used by client widget to find patchbay boxes '''
+        self.ui.framePatchbayFilters.setVisible(True)
+        self.ui.framePatchbayFilters.set_filter_text(text)
+
     def update_favorites_menu(self):
         self._favorites_menu.clear()
 
@@ -1253,10 +1514,14 @@ class MainWindow(QMainWindow):
 
         for favorite in self.session.favorite_list:
             act_app = self._favorites_menu.addAction(
-                ray.getAppIcon(favorite.icon, self), favorite.name)
+                get_app_icon(favorite.icon, self), favorite.name)
             act_app.setData([favorite.name, favorite.factory])
             act_app.triggered.connect(self.launch_favorite)
 
+        self._favorites_menu.setEnabled(
+            bool(enable and self.session.favorite_list))
+        self._build_systray_menu()
+
     def show_script_info(self, text):
         if self._script_info_dialog and self._script_info_dialog.should_be_removed():
             del self._script_info_dialog
@@ -1332,6 +1597,22 @@ class MainWindow(QMainWindow):
 
     def closeEvent(self, event):
         self.save_window_settings()
+        self.hidden_maximized = self.isMaximized()
+
+        if self._systray.isVisible() and self.session.is_running():
+            if not RS.is_hidden(RS.HD_SystrayClose):
+                dialog = child_dialogs.SystrayCloseDialog(self)
+                dialog.exec()
+
+                if not dialog.result():
+                    event.ignore()
+                    return
+
+                if dialog.not_again():
+                    RS.set_hidden(RS.HD_SystrayClose)
+
+            self.hide()
+            return
 
         if self._quit_app():
             QMainWindow.closeEvent(self, event)
@@ -1354,20 +1635,22 @@ class MainWindow(QMainWindow):
         QMainWindow.showEvent(self, event)
 
     def hideEvent(self, event):
+        self.hidden_maximized = self.isMaximized()
+
         if CommandLineArgs.under_nsm:
             if self.session.nsm_child is not None:
                 self.session.nsm_child.send_gui_state(False)
         QMainWindow.hideEvent(self, event)
-    
+
     def resizeEvent(self, event):
         if self._fullscreen_patchbay and not self.isFullScreen():
             self.toggle_scene_full_screen()
-        
+
         QMainWindow.resizeEvent(self, event)
-        
+
         new_button = self.ui.toolBar.widgetForAction(self.ui.actionNewSession)
         open_button = self.ui.toolBar.widgetForAction(self.ui.actionOpenSession)
-        
+
         if self.width() > 410:
             for button in (new_button, open_button):
                 button.setToolButtonStyle(Qt.ToolButtonTextBesideIcon)
