@@ -61,7 +61,9 @@ from .theme import Theme
 from .connect_menu import MainPortContextMenu
 from .utils import (
     CanvasGetFullPortName,
+    CanvasGetPortGroupPortList,
     CanvasGetPortConnectionList,
+    CanvasGetPortGroupName,
     CanvasGetPortGroupPosition,
     CanvasGetPortPrintName,
     CanvasConnectionMatches,
@@ -87,6 +89,10 @@ class CanvasPort(QGraphicsItem):
         self.m_port_name = port_name
         self.m_portgrp_id = 0
         self.m_is_alternate = is_alternate
+        self.m_print_name = port_name
+        self.m_print_name_right = ''
+        self.m_name_truncked = False
+        self.m_trunck_sep = '⠿'
 
         # Base Variables
         self.m_port_width = 15
@@ -135,11 +141,11 @@ class CanvasPort(QGraphicsItem):
     def is_connectable_to(self, other, accept_same_port_mode=False)->bool:
         if self.m_port_type != other.getPortType():
             return False
-        
+
         if not accept_same_port_mode:
             if self.m_port_mode == other.getPortMode():
                 return False
-        
+
         if self.m_port_type == PORT_TYPE_AUDIO_JACK:
             if other.getPortMode() == self.m_port_mode:
                 return bool(self.isAlternate() == other.isAlternate())
@@ -150,12 +156,12 @@ class CanvasPort(QGraphicsItem):
                 if self.isAlternate():
                     return other.isAlternate()
                 return True
-            
+
             if self.m_port_mode == PORT_MODE_INPUT:
                 if self.isAlternate():
                     return True
                 return not other.isAlternate()
-        
+
         return True
 
     def getFullPortName(self):
@@ -173,21 +179,69 @@ class CanvasPort(QGraphicsItem):
 
     def setPortGroupId(self, portgrp_id):
         self.m_portgrp_id = portgrp_id
-        #self.update()
 
     def setPortName(self, port_name):
-        if QFontMetrics(self.m_port_font).width(port_name) < QFontMetrics(self.m_port_font).width(self.m_port_name):
-            QTimer.singleShot(0, canvas.scene.update)
+        # if (QFontMetrics(self.m_port_font).width(port_name)
+        #         < QFontMetrics(self.m_port_font).width(self.m_port_name)):
+        #     QTimer.singleShot(0, canvas.scene.update)
 
         self.m_port_name = port_name
-        self.update()
+        # self.update()
+
+    def get_width_for_text(self, text: str):
+        return QFontMetrics(self.m_port_font).width(text)
+
+    def reduce_print_text(self, print_text: str):
+        pass
 
     def setPortWidth(self, port_width):
-        if port_width < self.m_port_width:
-            QTimer.singleShot(0, canvas.scene.update)
+        #if port_width < self.m_port_width:
+            #QTimer.singleShot(0, canvas.scene.update)
 
         self.m_port_width = port_width
-        self.update()
+        #self.update()
+
+    def set_print_name(self, print_name:str, width_limited: int):
+        self.m_print_name = print_name
+        self.m_name_truncked = False
+
+        if width_limited:
+            sizer = QFontMetrics(self.m_port_font)
+
+            if sizer.width(self.m_print_name) > width_limited:
+                name_len = len(self.m_print_name)
+                middle = int(name_len / 2)
+                left_text = self.m_print_name[:middle]
+                middle_text = self.m_trunck_sep
+                right_text = self.m_print_name[middle + 1:]
+                left_size = sizer.width(left_text)
+                middle_size = sizer.width(middle_text)
+                right_size = sizer.width(right_text)
+
+                while left_size + middle_size + right_size > width_limited:
+                    if left_size > right_size:
+                        left_text = left_text[:-1]
+                        left_size = sizer.width(left_text)
+                    else:
+                        right_text = right_text[1:]
+                        right_size = sizer.width(right_text)
+                        
+                    if not (left_text or right_text):
+                        break
+
+                self.m_print_name = left_text
+                self.m_print_name_right = right_text
+                self.m_name_truncked = True
+
+    def get_text_width(self):
+        sizer = QFontMetrics(self.m_port_font)
+
+        if self.m_name_truncked:
+            return (sizer.width(self.m_print_name)
+                    + sizer.width(self.m_trunck_sep)
+                    + sizer.width(self.m_print_name_right))
+            
+        return sizer.width(self.m_print_name)
 
     def resetLineMovPositions(self):
         for i in range(len(self.m_line_mov_list)):
@@ -223,6 +277,9 @@ class CanvasPort(QGraphicsItem):
 
         CanvasCallback(ACTION_PORTGROUP_ADD, 0, 0, data)
 
+    def set_paint_attributes(self, print_name:str, port_width:int):
+        pass
+
     def connectToHover(self):
         if self.m_hover_item:
             hover_port_id_list = []
@@ -297,6 +354,11 @@ class CanvasPort(QGraphicsItem):
         QGraphicsItem.hoverLeaveEvent(self, event)
 
     def mousePressEvent(self, event):
+        if canvas.scene.get_zoom_scale() <= 0.4:
+            # prefer move box if zoom is too low
+            event.ignore()
+            return
+        
         if event.button() == Qt.LeftButton:
             self.m_hover_item = None
             self.m_mouse_down = True
@@ -538,6 +600,11 @@ class CanvasPort(QGraphicsItem):
         QGraphicsItem.mouseReleaseEvent(self, event)
 
     def contextMenuEvent(self, event):
+        if canvas.scene.get_zoom_scale() <= 0.4:
+            # prefer move box if zoom is too low
+            event.ignore()
+            return
+        
         if canvas.is_line_mov:
             return
 
@@ -624,16 +691,20 @@ class CanvasPort(QGraphicsItem):
     def boundingRect(self):
         if self.m_portgrp_id:
             if self.m_port_mode == PORT_MODE_INPUT:
-                return QRectF(0, 0, canvas.theme.port_in_portgrp_width, self.m_port_height)
+                return QRectF(0, 0, self.m_port_width, self.m_port_height)
             else:
-                return QRectF(self.m_port_width +12 - canvas.theme.port_in_portgrp_width,
-                              0, canvas.theme.port_in_portgrp_width, self.m_port_height)
+                return QRectF(12, 0,
+                              self.m_port_width, self.m_port_height)
         else:
             return QRectF(0, 0, self.m_port_width + 12, self.m_port_height)
 
     def paint(self, painter, option, widget):
+        if canvas.scene.loading_items:
+            return
+        
         painter.save()
-        painter.setRenderHint(QPainter.Antialiasing, bool(options.antialiasing == ANTIALIASING_FULL))
+        painter.setRenderHint(QPainter.Antialiasing,
+                              bool(options.antialiasing == ANTIALIASING_FULL))
 
         selected = self.isSelected()
         theme = canvas.theme
@@ -673,7 +744,8 @@ class CanvasPort(QGraphicsItem):
         lineHinting = poly_pen.widthF() / 2
 
         poly_locx = [0, 0, 0, 0, 0, 0]
-        poly_corner_xhinting = (float(canvas.theme.port_height)/2) % floor(float(canvas.theme.port_height)/2)
+        poly_corner_xhinting = ((float(canvas.theme.port_height)/2)
+                                % floor(float(canvas.theme.port_height)/2))
         if poly_corner_xhinting == 0:
             poly_corner_xhinting = 0.5 * (1 - 7 / (float(canvas.theme.port_height)/2))
 
@@ -689,16 +761,17 @@ class CanvasPort(QGraphicsItem):
                 poly_locx[2] = self.m_port_width + 12 - poly_corner_xhinting
                 poly_locx[3] = self.m_port_width + 5 - lineHinting
                 poly_locx[4] = lineHinting
-                poly_locx[5] = canvas.theme.port_in_portgrp_width
+                poly_locx[5] = self.m_port_width
             elif canvas.theme.port_mode == Theme.THEME_PORT_SQUARE or is_cv_port:
                 poly_locx[0] = lineHinting
                 poly_locx[1] = self.m_port_width + 5 - lineHinting
                 poly_locx[2] = self.m_port_width + 5 - lineHinting
                 poly_locx[3] = self.m_port_width + 5 - lineHinting
                 poly_locx[4] = lineHinting
-                poly_locx[5] = canvas.theme.port_in_portgrp_width
+                poly_locx[5] = self.m_port_width
             else:
-                qCritical("PatchCanvas::CanvasPort.paint() - invalid theme port mode '%s'" % canvas.theme.port_mode)
+                qCritical("PatchCanvas::CanvasPort.paint() - invalid theme port mode '%s'"
+                          % canvas.theme.port_mode)
                 return
 
         elif self.m_port_mode == PORT_MODE_OUTPUT:
@@ -710,37 +783,32 @@ class CanvasPort(QGraphicsItem):
                 poly_locx[2] = 0 + poly_corner_xhinting
                 poly_locx[3] = 7 + lineHinting
                 poly_locx[4] = self.m_port_width + 12 - lineHinting
-                poly_locx[5] = self.m_port_width + 12 - canvas.theme.port_in_portgrp_width - lineHinting
+                poly_locx[5] = 12 - lineHinting
             elif canvas.theme.port_mode == Theme.THEME_PORT_SQUARE or is_cv_port:
                 poly_locx[0] = self.m_port_width + 12 - lineHinting
                 poly_locx[1] = 5 + lineHinting
                 poly_locx[2] = 5 + lineHinting
                 poly_locx[3] = 5 + lineHinting
                 poly_locx[4] = self.m_port_width + 12 - lineHinting
-                poly_locx[5] = self.m_port_width + 12 -canvas.theme.port_in_portgrp_width - lineHinting
+                poly_locx[5] = 12 - lineHinting
             else:
-                qCritical("PatchCanvas::CanvasPort.paint() - invalid theme port mode '%s'" % canvas.theme.port_mode)
+                qCritical("PatchCanvas::CanvasPort.paint() - invalid theme port mode '%s'"
+                          % canvas.theme.port_mode)
                 return
 
         else:
-            qCritical("PatchCanvas::CanvasPort.paint() - invalid port mode '%s'" % port_mode2str(self.m_port_mode))
+            qCritical("PatchCanvas::CanvasPort.paint() - invalid port mode '%s'"
+                      % port_mode2str(self.m_port_mode))
             return
 
-        #if is_cv_port and not selected:
-            #poly_color = QColor(170, 140, 150)
-
-        #if self.m_is_alternate:
-            #poly_color = poly_color.lighter(230)
-            #poly_pen.setColor(poly_pen.color().darker(110))
-            #text_pen.setColor(text_pen.color()) #.darker(150))
-            #conn_pen.setColor(conn_pen.color()) #.darker(150))
-
         polygon = QPolygonF()
 
         if self.m_portgrp_id:
             first_of_portgrp = False
             last_of_portgrp = False
 
+            # look in portgroup if port is the first,
+            # the last, or not.
             for portgrp in canvas.portgrp_list:
                 if portgrp.portgrp_id == self.m_portgrp_id:
                     if self.m_port_id == portgrp.port_id_list[0]:
@@ -774,7 +842,8 @@ class CanvasPort(QGraphicsItem):
             portRect = polygon.boundingRect().adjusted(
                 -lineHinting+1, -lineHinting+1, lineHinting-1, lineHinting-1)
             portPos = portRect.topLeft()
-            painter.drawTiledPixmap(portRect, canvas.theme.port_bg_pixmap, portPos)
+            painter.drawTiledPixmap(
+                portRect, canvas.theme.port_bg_pixmap, portPos)
         else:
             port_gradient = QLinearGradient(0, 0, 0, self.m_port_height)
 
@@ -801,11 +870,15 @@ class CanvasPort(QGraphicsItem):
 
                 y_line = canvas.theme.port_height / 2.0
                 if self.m_port_mode == PORT_MODE_OUTPUT:
-                    painter.drawLine(0, y_line, poly_locx[1], y_line)
+                    painter.drawLine(
+                        QPointF(0.0, y_line),
+                        QPointF(float(poly_locx[1]), y_line))
                 elif self.m_port_mode == PORT_MODE_INPUT:
-                    painter.drawLine(self.m_port_width + 5, y_line, self.m_port_width + 12, y_line)
+                    painter.drawLine(
+                        QPointF(self.m_port_width + 5.0, y_line),
+                        QPointF(self.m_port_width + 12.0, y_line))
             else:
-                # draw the little circle for a2j port
+                # draw the little circle for a2j (or MidiBridge) port
                 poly_pen.setWidthF(1.000001)
                 painter.setBrush(canvas.theme.box_bg_1)
 
@@ -821,24 +894,32 @@ class CanvasPort(QGraphicsItem):
         painter.setPen(text_pen)
         painter.setFont(self.m_port_font)
 
+        sizer = QFontMetrics(self.m_port_font)
+        sep_width = sizer.width(self.m_trunck_sep)
+
         if self.m_portgrp_id:
-            print_name = CanvasGetPortPrintName(
-                            self.m_group_id, self.m_port_id,
-                            self.m_portgrp_id)
-            print_name_size = QFontMetrics(self.m_port_font).width(print_name)
+            print_name_size = self.get_text_width()
+
             if self.m_port_mode == PORT_MODE_OUTPUT:
+                text_pos = QPointF(self.m_port_width + 9 - print_name_size,
+                                   canvas.theme.port_text_ypos)
 
-                text_pos = QPointF(self.m_port_width + 9 - print_name_size, canvas.theme.port_text_ypos)
-
-            if print_name_size > (canvas.theme.port_in_portgrp_width - 4):
+            if print_name_size > (self.m_port_width - 4):
                 painter.setPen(QPen(port_gradient, 3))
-                painter.drawLine(poly_locx[5], 3, poly_locx[5], canvas.theme.port_height - 3)
+                painter.drawLine(
+                    QPointF(float(poly_locx[5]), 3.0),
+                    QPointF(float(poly_locx[5]), canvas.theme.port_height - 3.0))
                 painter.setPen(text_pen)
                 painter.setFont(self.m_port_font)
-            painter.drawText(text_pos, print_name)
 
-        else:
-            painter.drawText(text_pos, self.m_port_name)
+        painter.drawText(text_pos, self.m_print_name)
+        
+        if self.m_name_truncked:
+            sep_x = text_pos.x() + sizer.width(self.m_print_name)
+            
+            painter.drawText(QPointF(sep_x + sep_width, text_pos.y()), self.m_print_name_right)
+            painter.setPen(poly_pen)
+            painter.drawText(QPointF(sep_x, text_pos.y() + 1), self.m_trunck_sep)
 
         if canvas.theme.idx == Theme.THEME_OOSTUDIO and canvas.theme.port_bg_pixmap:
             painter.setPen(Qt.NoPen)
