@@ -87,6 +87,13 @@ class CanvasPortGroup(QGraphicsItem):
         self.m_portgrp_font.setPixelSize(canvas.theme.port_font_size)
         self.m_portgrp_font.setWeight(canvas.theme.port_font_state)
 
+        self.m_ports_width = canvas.theme.port_in_portgrp_width
+        self.m_print_name = ''
+        self.m_normal_print_name = '' # same as m_print_name but not reduced
+        self.m_print_name_right = ''
+        self.m_name_truncked = False
+        self.m_trunck_sep = '⠿'
+
         self.m_line_mov_list = []
         self.m_dotcon_list = []
         self.m_last_rclick_item = None
@@ -113,11 +120,11 @@ class CanvasPortGroup(QGraphicsItem):
     def is_connectable_to(self, other, accept_same_port_mode=False)->bool:
         if self.m_port_type != other.getPortType():
             return False
-        
+
         if not accept_same_port_mode:
             if self.m_port_mode == other.getPortMode():
                 return False
-        
+
         if self.m_port_type == PORT_TYPE_AUDIO_JACK:
             if other.getPortMode() == self.m_port_mode:
                 return bool(self.isAlternate() == other.isAlternate())
@@ -128,12 +135,12 @@ class CanvasPortGroup(QGraphicsItem):
                 if self.isAlternate():
                     return other.isAlternate()
                 return True
-            
+
             if self.m_port_mode == PORT_MODE_INPUT:
                 if self.isAlternate():
                     return True
                 return not other.isAlternate()
-        
+
         return True
 
     def getGroupId(self):
@@ -158,11 +165,60 @@ class CanvasPortGroup(QGraphicsItem):
         return CanvasPortGroupType
 
     def setPortGroupWidth(self, portgrp_width):
-        if portgrp_width < self.m_portgrp_width:
-            QTimer.singleShot(0, canvas.scene.update)
+        #if portgrp_width < self.m_portgrp_width:
+            #QTimer.singleShot(0, canvas.scene.update)
 
         self.m_portgrp_width = portgrp_width
-        self.update()
+        #self.update()
+
+    def set_ports_width(self, ports_width:int):
+        self.m_ports_width = ports_width
+
+    def set_print_name(self, print_name:str, width_limited: int):
+        self.m_print_name = print_name
+        self.m_normal_print_name = print_name
+        self.m_name_truncked = False
+
+        if width_limited:
+            sizer = QFontMetrics(self.m_portgrp_font)
+            
+            if sizer.width(self.m_print_name) > width_limited:
+                name_len = len(self.m_print_name)
+                middle = int(name_len / 2)
+                left_text = self.m_print_name[:middle]
+                middle_text = self.m_trunck_sep
+                right_text = self.m_print_name[middle + 1:]
+                left_size = sizer.width(left_text)
+                middle_size = sizer.width(middle_text)
+                right_size = sizer.width(right_text)
+
+                while left_size + middle_size + right_size > width_limited:
+                    if left_size > right_size:
+                        left_text = left_text[:-1]
+                        left_size = sizer.width(left_text)
+                    else:
+                        right_text = right_text[1:]
+                        right_size = sizer.width(right_text)
+                    
+                    if not (left_text or right_text):
+                        break
+
+                self.m_print_name = left_text
+                self.m_print_name_right = right_text
+                self.m_name_truncked = True
+
+    def reduce_print_name(self, width_limited:int):
+        self.set_print_name(self.m_normal_print_name, width_limited)
+
+    def get_text_width(self):
+        sizer = QFontMetrics(self.m_portgrp_font)
+
+        if self.m_name_truncked:
+            return (sizer.width(self.m_print_name)
+                    + sizer.width(self.m_trunck_sep)
+                    + sizer.width(self.m_print_name_right))
+            
+        return sizer.width(self.m_print_name)
 
     def resetDotLines(self):
         for connection in self.m_dotcon_list:
@@ -302,6 +358,11 @@ class CanvasPortGroup(QGraphicsItem):
         QGraphicsItem.hoverLeaveEvent(self, event)
 
     def mousePressEvent(self, event):
+        if canvas.scene.get_zoom_scale() <= 0.4:
+            # prefer move box if zoom is too low
+            event.ignore()
+            return
+        
         if event.button() == Qt.LeftButton:
             self.m_hover_item = None
             self.m_mouse_down = True
@@ -571,8 +632,12 @@ class CanvasPortGroup(QGraphicsItem):
             canvas.is_line_mov = False
         QGraphicsItem.mouseReleaseEvent(self, event)
 
-
     def contextMenuEvent(self, event):
+        if canvas.scene.get_zoom_scale() <= 0.4:
+            # prefer move box if zoom is too low
+            event.ignore()
+            return
+        
         if canvas.is_line_mov:
             return
 
@@ -613,6 +678,9 @@ class CanvasPortGroup(QGraphicsItem):
                           canvas.theme.port_height * len(self.m_port_id_list))
 
     def paint(self, painter, option, widget):
+        if canvas.scene.loading_items:
+            return
+        
         painter.save()
         painter.setRenderHint(
             QPainter.Antialiasing, bool(options.antialiasing == ANTIALIASING_FULL))
@@ -630,26 +698,28 @@ class CanvasPortGroup(QGraphicsItem):
 
             for port in canvas.port_list:
                 if port.port_id in self.m_port_id_list:
-                    port_print_name = CanvasGetPortPrintName(port.group_id,
-                                        port.port_id, self.m_portgrp_id)
+                    port_print_name = CanvasGetPortPrintName(
+                        port.group_id, port.port_id, self.m_portgrp_id)
                     port_in_p_width = QFontMetrics(self.m_portgrp_font).width(port_print_name) + 3
                     port_width = max(port_width, port_in_p_width)
 
-            text_pos = QPointF(port_width + 3, canvas.theme.port_text_ypos \
-                       + (canvas.theme.port_height * (len(self.m_port_id_list) -1)/2))
+            text_pos = QPointF(
+                self.m_ports_width + 3,
+                canvas.theme.port_text_ypos
+                    + (canvas.theme.port_height * (len(self.m_port_id_list) -1)/2))
 
             if canvas.theme.port_mode == Theme.THEME_PORT_POLYGON:
-                poly_locx[0] = canvas.theme.port_in_portgrp_width - lineHinting
-                poly_locx[1] = self.m_portgrp_width + 5 + lineHinting
-                poly_locx[2] = self.m_portgrp_width + 12 + lineHinting
-                poly_locx[3] = self.m_portgrp_width + 5 + lineHinting
-                poly_locx[4] = canvas.theme.port_in_portgrp_width - lineHinting
+                poly_locx[0] = self.m_ports_width - lineHinting
+                poly_locx[1] = self.m_portgrp_width + 3 + lineHinting
+                poly_locx[2] = self.m_portgrp_width + 10 + lineHinting
+                poly_locx[3] = self.m_portgrp_width + 3 + lineHinting
+                poly_locx[4] = self.m_ports_width - lineHinting
             elif canvas.theme.port_mode == Theme.THEME_PORT_SQUARE:
-                poly_locx[0] = canvas.theme.port_in_portgrp_width - lineHinting
+                poly_locx[0] = self.m_ports_width - lineHinting
                 poly_locx[1] = self.m_portgrp_width + 5 + lineHinting
                 poly_locx[2] = self.m_portgrp_width + 5 + lineHinting
                 poly_locx[3] = self.m_portgrp_width + 5 + lineHinting
-                poly_locx[4] = canvas.theme.port_in_portgrp_width - lineHinting
+                poly_locx[4] = self.m_ports_width - lineHinting
             else:
                 qCritical("PatchCanvas::CanvasPortGroup.paint() - invalid theme port mode '%s'"
                           % canvas.theme.port_mode)
@@ -661,19 +731,21 @@ class CanvasPortGroup(QGraphicsItem):
 
             if canvas.theme.port_mode == Theme.THEME_PORT_POLYGON:
                 poly_locx[0] = self.m_portgrp_width + 12 \
-                               - canvas.theme.port_in_portgrp_width - lineHinting
+                               - self.m_ports_width - lineHinting
                 poly_locx[1] = 7 + lineHinting
                 poly_locx[2] = 0 + lineHinting
                 poly_locx[3] = 7 + lineHinting
-                poly_locx[4] = self.m_portgrp_width + 12 - canvas.theme.port_in_portgrp_width - lineHinting
+                poly_locx[4] = self.m_portgrp_width + 12 - self.m_ports_width - lineHinting
             elif canvas.theme.port_mode == Theme.THEME_PORT_SQUARE:
-                poly_locx[0] = self.m_portgrp_width + 12 - canvas.theme.port_in_portgrp_width - lineHinting
+                poly_locx[0] = self.m_portgrp_width + 12 - self.m_ports_width - lineHinting
                 poly_locx[1] = 5 + lineHinting
                 poly_locx[2] = 5 + lineHinting
                 poly_locx[3] = 5 + lineHinting
-                poly_locx[4] = self.m_portgrp_width + 12 - canvas.theme.port_in_portgrp_width - lineHinting
+                poly_locx[4] = self.m_portgrp_width + 12 - self.m_ports_width - lineHinting
             else:
-                qCritical("PatchCanvas::CanvasPortGroup.paint() - invalid theme port mode '%s'" % canvas.theme.port_mode)
+                qCritical(
+                    "PatchCanvas::CanvasPortGroup.paint() - invalid theme port mode '%s'"
+                    % canvas.theme.port_mode)
                 return
 
         else:
@@ -712,9 +784,16 @@ class CanvasPortGroup(QGraphicsItem):
 
         painter.setPen(text_pen)
         painter.setFont(self.m_portgrp_font)
-        portgrp_name = CanvasGetPortGroupName(self.m_group_id,
-                                              self.m_port_id_list)
-        painter.drawText(text_pos, portgrp_name)
+        painter.drawText(text_pos, self.m_print_name)
+        if self.m_name_truncked:
+            sizer = QFontMetrics(self.m_portgrp_font)
+            sep_x = text_pos.x() + sizer.width(self.m_print_name)
+            sep_width = sizer.width(self.m_trunck_sep)
+
+            painter.drawText(QPointF(sep_x + sep_width, text_pos.y()),
+                             self.m_print_name_right)
+            painter.setPen(poly_pen)
+            painter.drawText(QPointF(sep_x, text_pos.y() + 1), self.m_trunck_sep)
 
         painter.restore()
 
