@@ -55,7 +55,9 @@ class Theme(object):
         self.box_header_spacing = 0
 
         self.box_text_ypos = 20
+        self.box_text_hw = QPen(QColor(180, 180, 180), 0)
 
+        self.box_pen_hw = QPen(QColor(56, 57, 58), 1, Qt.SolidLine)
         self.box_bg_type = self.THEME_BG_GRADIENT
         self.box_use_icon = True
 
@@ -64,11 +66,16 @@ class Theme(object):
 
         self.port_mode = self.THEME_PORT_POLYGON
 
-        self.port_in_portgrp_width = 18
+        self.port_in_portgrp_width = 19
         self.port_height = 16
         self.port_offset = 0
         self.port_spacing = 2
         self.port_spacingT = 2
+        
+        self.box_spacing = 4
+        self.box_spacing_hor = 24
+        self.magnet = 12
+        self.hardware_rack_width = 5
 
         self.setTheme(idx)
 
@@ -81,7 +88,7 @@ class Theme(object):
             self.box_pen = QPen(QColor(76, 77, 78), 1, Qt.SolidLine)
             self.box_pen_sel = QPen(QColor(206, 207, 208), 1, Qt.DashLine)
             self.box_bg_1 = QColor(32, 34, 35)
-            self.box_bg_2 = QColor(43, 47, 48)
+            self.box_bg_2 = QColor(38, 40, 41)
             self.box_shadow = QColor(89, 89, 89, 180)
 
             self.box_text = QPen(QColor(240, 240, 240), 0)
@@ -158,7 +165,7 @@ class Theme(object):
             self.box_pen = QPen(QColor(76, 77, 78), 1, Qt.SolidLine)
             self.box_pen_sel = QPen(QColor(206, 207, 208), 1, Qt.DashLine)
             self.box_bg_1 = QColor(32, 34, 35)
-            self.box_bg_2 = QColor(43, 47, 48)
+            self.box_bg_2 = QColor(38, 40, 41)
             self.box_shadow = QColor(89, 89, 89, 180)
 
             self.box_text = QPen(QColor(210, 210, 210), 0)
@@ -240,7 +247,7 @@ class Theme(object):
             self.box_pen = QPen(QColor(76, 77, 78), 1, Qt.SolidLine)
             self.box_pen_sel = QPen(QColor(206, 207, 208), 1, Qt.DashLine)
             self.box_bg_1 = QColor(32, 34, 35)
-            self.box_bg_2 = QColor(43, 47, 48)
+            self.box_bg_2 = QColor(38, 40, 41)
             self.box_shadow = QColor(89, 89, 89, 180)
 
             self.box_text = QPen(QColor(240, 240, 240), 0)
