@@ -23,14 +23,20 @@ if QT_VERSION < (5, 6):
         "WARNING: You are using a version of QT older than 5.6.\n"
         + "You won't be warned if a process can't be launch.\n")
 
-VERSION = "0.9.1"
+VERSION = "0.12.2"
 
 APP_TITLE = 'RaySession'
 DEFAULT_SESSION_ROOT = "%s/Ray Sessions" % os.getenv('HOME')
 SCRIPTS_DIR = 'ray-scripts'
 NOTES_PATH = 'ray-notes'
-factory_session_templates = (
+FACTORY_SESSION_TEMPLATES = (
     'with_jack_patch', 'with_jack_config', 'scripted')
+RAYNET_BIN = 'ray-network'
+
+GIT_IGNORED_EXTENSIONS = ".wav .flac .ogg .mp3 .mp4 .avi .mkv .peak .m4a .pdf"
+
+GROUP_CONTEXT_AUDIO = 0x01
+GROUP_CONTEXT_MIDI = 0x02
 
 class PrefixMode:
     CUSTOM = 0
@@ -38,6 +44,10 @@ class PrefixMode:
     SESSION_NAME = 2
 
 
+class JackNaming:
+    SHORT = 0
+    LONG = 1
+
 class ClientStatus:
     STOPPED = 0
     LAUNCH = 1
@@ -96,6 +106,8 @@ class Option:
     HAS_GIT = 0x020
     SNAPSHOTS = 0x040
     SESSION_SCRIPTS = 0x080
+    GUI_STATES = 0x100
+
 
 class Err:
     OK = 0
@@ -152,18 +164,26 @@ class Template:
     CLIENT_SAVE = 6
     CLIENT_LOAD = 7
 
+
 class SwitchState:
     NONE = 0
     RESERVED = 1
     NEEDED = 2
     DONE = 3
 
+
 class WindowManager:
     NONE = 0
     X = 1
     WAYLAND = 2
 
 
+class Systray:
+    OFF = 0
+    SESSION_ONLY = 1
+    ALWAYS = 2
+
+
 class Favorite():
     def __init__(self, name, icon, factory):
         self.name = name
@@ -171,6 +191,23 @@ class Favorite():
         self.factory = factory
 
 
+class ScriptFile:
+    PREVENT = 0x0
+    PARENT = 0x1
+    LOAD = 0x2
+    SAVE = 0x4
+    CLOSE = 0x8
+
+    @classmethod
+    def by_string(cls, action:str)->int:
+        if action == 'load':
+            return cls.LOAD
+        if action == 'save':
+            return cls.SAVE
+        if action == 'close':
+            return cls.CLOSE
+        return cls.PREVENT
+
 debug = False
 
 
@@ -178,11 +215,7 @@ def ifDebug(string):
     if debug:
         sys.stderr.write("%s\n" % string)
 
-def setDebug(bool_debug: bool):
-    global debug
-    debug = bool_debug
-
-def versionToTuple(version_str):
+def version_to_tuple(version_str):
     version_list = []
     for c in version_str.split('.'):
         if not c.isdigit():
@@ -191,7 +224,7 @@ def versionToTuple(version_str):
 
     return tuple(version_list)
 
-def addSelfBinToPath():
+def add_self_bin_to_path():
     # Add raysession/src/bin to $PATH to can use ray executables after make
     # Warning, will works only if link to this file is in RaySession/*/*/*.py
     this_path = os.path.realpath(os.path.dirname(os.path.realpath(__file__)))
@@ -199,8 +232,7 @@ def addSelfBinToPath():
     if not os.environ['PATH'].startswith("%s:" % bin_path):
         os.environ['PATH'] = "%s:%s" % (bin_path, os.environ['PATH'])
 
-
-def getListInSettings(settings, path):
+def get_list_in_settings(settings, path):
     # getting a QSettings value of list type seems to not works the same way
     # on all machines
     try:
@@ -213,44 +245,8 @@ def getListInSettings(settings, path):
 
     return settings_list
 
-def getGitIgnoredExtensions():
-    return ".wav .flac .ogg .mp3 .mp4 .avi .mkv .peak .m4a .pdf"
-
-def isPidChildOf(child_pid, parent_pid):
-    if child_pid < parent_pid:
-        return False
-
-    ppid = child_pid
-
-    while ppid > parent_pid:
-        try:
-            proc_file = open('/proc/%i/status' % ppid, 'r')
-            proc_contents = proc_file.read()
-        except BaseException:
-            return False
-
-        for line in proc_contents.split('\n'):
-            if line.startswith('PPid:'):
-                ppid_str = line.rpartition('\t')[2]
-                if ppid_str.isdigit():
-                    ppid = int(ppid_str)
-                    break
-        else:
-            return False
-
-    #while ppid != parent_pid and ppid > 1 and ppid != this_pid:
-        #try:
-            #ppid = int(subprocess.check_output(
-                #['ps', '-o', 'ppid=', '-p', str(ppid)]))
-        #except BaseException:
-            #return False
-
-    if ppid == parent_pid:
-        return True
-
-    return False
-
-def isGitTaggable(string):
+def is_git_taggable(string)->bool:
+    ''' know if a string can be a git tag, not used currently '''
     if not string:
         return False
 
@@ -273,12 +269,19 @@ def isGitTaggable(string):
 
     return True
 
-def highlightText(string):
-    if "'" in string:
-        return '"%s"' % string
-    return "'%s'" % string
+def is_valid_full_path(path: str)->bool:
+    if not path.startswith('/'):
+        return False
 
-def isOscPortFree(port):
+    for forbidden in ('//', '/./', '/../'):
+        if forbidden in path:
+            return False
+
+    if path.endswith(('/.', '/..')):
+        return False
+    return True
+
+def is_osc_port_free(port:int):
     try:
         testport = Server(port)
     except BaseException:
@@ -288,37 +291,35 @@ def isOscPortFree(port):
     return True
 
 
-def getFreeOscPort(default=16187):
-    # get a free OSC port for daemon, start from default
+def get_free_osc_port(default=16187):
+    ''' get a free OSC port for daemon, start from default '''
 
     if default >= 65536:
         default = 16187
 
     daemon_port = default
-    UsedPort = True
+    used_port = True
     testport = None
 
-    while UsedPort:
+    while used_port:
         try:
             testport = Server(daemon_port)
-            UsedPort = False
+            used_port = False
         except BaseException:
             daemon_port += 1
-            UsedPort = True
+            used_port = True
 
     del testport
     return daemon_port
 
-
-def isValidOscUrl(url):
+def is_valid_osc_url(url:str):
     try:
         address = liblo.Address(url)
         return True
     except BaseException:
         return False
 
-
-def getLibloAddress(url):
+def get_liblo_address(url):
     valid_url = False
     try:
         address = liblo.Address(url)
@@ -336,7 +337,7 @@ def getLibloAddress(url):
             msg = "%r is an unknown osc url" % url
             raise argparse.ArgumentTypeError(msg)
 
-def getLibloAddressFromPort(port):
+def get_liblo_address_from_port(port:int):
     try:
         port = int(port)
     except:
@@ -361,7 +362,7 @@ def getLibloAddressFromPort(port):
             msg = "%i is an unknown osc port" % port
             raise argparse.ArgumentTypeError(msg)
 
-def areSameOscPort(url1, url2):
+def are_same_osc_port(url1, url2):
     if url1 == url2:
         return True
     try:
@@ -373,13 +374,12 @@ def areSameOscPort(url1, url2):
     if address1.port != address2.port:
         return False
 
-    if areOnSameMachine(url1, url2):
+    if are_on_same_machine(url1, url2):
         return True
 
     return False
 
-
-def areOnSameMachine(url1, url2):
+def are_on_same_machine(url1, url2):
     if url1 == url2:
         return True
 
@@ -393,109 +393,51 @@ def areOnSameMachine(url1, url2):
         return True
 
     try:
-        if ((socket.gethostbyname(address1.hostname) in ('127.0.0.1', '127.0.1.1')) and (
-                socket.gethostbyname(address2.hostname) in ('127.0.0.1', '127.0.1.1'))):
+        if (socket.gethostbyname(address1.hostname)
+                    in ('127.0.0.1', '127.0.1.1')
+                and socket.gethostbyname(address2.hostname)
+                    in ('127.0.0.1', '127.0.1.1')):
             return True
 
         if socket.gethostbyaddr(
                 address1.hostname) == socket.gethostbyaddr(
                 address2.hostname):
             return True
-    except BaseException:
-        try:
-            ips = subprocess.check_output(['hostname', '-I']).decode()
-            ip = ips.split(' ')[0]
 
-            if ip.count('.') != 3:
-                return False
+        ip = Machine192.get()
 
-            if ip not in (address1.hostname, address2.hostname):
-                return False
-
-            try:
-                if socket.gethostbyname(
-                        address1.hostname) in (
-                        '127.0.0.1',
-                        '127.0.1.1'):
-                    if address2.hostname == ip:
-                        return True
-            except BaseException:
-                if socket.gethostbyname(
-                        address2.hostname) in (
-                        '127.0.0.1',
-                        '127.0.1.1'):
-                    if address1.hostname == ip:
-                        return True
-
-        except BaseException:
+        if ip not in (address1.hostname, address2.hostname):
             return False
 
+        if (ip == socket.gethostbyname(address1.hostname)
+                == socket.gethostbyname(address2.hostname)):
+            # on some systems (as fedora),
+            # socket.gethostbyname returns a 192.168.. url
+            return True
+
+        if (socket.gethostbyname(address1.hostname)
+                in ('127.0.0.1', '127.0.1.1')):
+            if address2.hostname == ip:
+                return True
+
+        if (socket.gethostbyname(address2.hostname)
+                in ('127.0.0.1', '127.0.1.1')):
+            if address1.hostname == ip:
+                return True
+
+    except BaseException:
         return False
 
     return False
 
-
-def getUrl192(url):
-    try:
-        ips = subprocess.check_output(['hostname', '-I']).decode()
-        ip = ips.split(' ')[0]
-    except BaseException:
-        return url
-
-    if ip.count('.') != 3:
-        return url
-
-    suffix_port = url.rpartition(':')[2]
-    return "osc.udp://%s:%s" % (ip, suffix_port)
-
-
-def getThis192():
-    global machine192
-
-    if 'machine192' in globals():
-        return machine192
-
-    try:
-        ips = subprocess.check_output(['hostname', '-I']).decode()
-        ip = ips.split(' ')[0]
-        machine192 = ip
-        return ip
-    except BaseException:
-        return ''
-
-def getMachine192(hostname=None):
-    if hostname is None:
-        return getThis192()
-
-    if hostname in ('localhost', socket.gethostname()):
-        return getThis192()
-
-    return socket.gethostbyname(hostname)
-
-def getMachine192ByUrl(url):
-    try:
-        addr = Address(url)
-    except BaseException:
-        return ''
-
-    hostname = addr.hostname
-    del addr
-
-    return getMachine192(hostname)
-
-def getNetUrl(port):
-    try:
-        ips = subprocess.check_output(['hostname', '-I']).decode()
-        ip = ips.split(' ')[0]
-    except BaseException:
-        return ''
-
-    if ip.count('.') != 3:
+def get_net_url(port)->str:
+    ip = Machine192.get()
+    if not ip:
         return ''
 
     return "osc.udp://%s:%i/" % (ip, port)
 
-def shellLineToArgs(string):
+def shell_line_to_args(string:str)->list:
     try:
         args = shlex.split(string)
     except BaseException:
@@ -503,36 +445,19 @@ def shellLineToArgs(string):
 
     return args
 
-def areTheyAllString(args):
+def types_are_all_strings(types:str)->bool:
+    for char in types:
+        if char != 's':
+            return False
+    return True
+
+def are_they_all_strings(args:list)->bool:
     for arg in args:
         if not isinstance(arg, str):
             return False
     return True
 
-def getAppIcon(icon_name, widget):
-    dark = bool(
-        widget.palette().brush(
-            2, QPalette.WindowText).color().lightness() > 128)
-
-    icon = QIcon.fromTheme(icon_name)
-
-    if icon.isNull():
-        for ext in ('svg', 'svgz', 'png'):
-            filename = ":app_icons/%s.%s" % (icon_name, ext)
-            darkname = ":app_icons/dark/%s.%s" % (icon_name, ext)
-
-            if dark and QFile.exists(darkname):
-                filename = darkname
-
-            if QFile.exists(filename):
-                del icon
-                icon = QIcon()
-                icon.addFile(filename)
-                break
-
-    return icon
-
-def getWindowManager():
+def get_window_manager():
     if os.getenv('WAYLAND_DISPLAY'):
         return WindowManager.WAYLAND
 
@@ -541,7 +466,7 @@ def getWindowManager():
 
     return WindowManager.NONE
 
-def getFullPath(root, session_name):
+def get_full_path(root, session_name):
     spath = "%s%s%s" % (root, os.sep, session_name)
 
     if session_name.startswith(os.sep):
@@ -552,13 +477,56 @@ def getFullPath(root, session_name):
 
     return spath
 
-def protocolToStr(protocol: int)->str:
+def protocol_to_str(protocol: int)->str:
     if protocol == Protocol.RAY_HACK:
         return "Ray-Hack"
     if protocol == Protocol.RAY_NET:
-        return "Net-Session"
+        return "Ray-Net"
     return "NSM"
 
+def protocol_from_str(protocol_str: str)->int:
+    if protocol_str.lower() in ('ray_hack', 'ray-hack'):
+        return Protocol.RAY_HACK
+    elif protocol_str.lower() in ('ray_net', 'ray-net'):
+        return Protocol.RAY_NET
+    return Protocol.NSM
+
+
+class Machine192:
+    ip = ''
+    read_done = False
+    
+    @staticmethod
+    def read()->str:
+        try:
+            ips = subprocess.check_output(
+                ['ip', 'route', 'get', '1']).decode()
+            ip_line = ips.partition('\n')[0]
+            ip_end = ip_line.rpartition('src ')[2]
+            ip = ip_end.partition(' ')[0]
+
+        except BaseException:
+            try:
+                ips = subprocess.check_output(['hostname', '-I']).decode()
+                ip = ips.split(' ')[0]
+            except BaseException:
+                return ''
+
+        if ip.count('.') != 3:
+            return ''
+        
+        return ip
+    
+    @classmethod
+    def get(cls)->str:
+        if cls.read_done:
+            return cls.ip
+        
+        cls.ip = cls.read()
+        cls.read_done = True
+
+        return cls.ip
+
 
 class ClientData:
     client_id = ''
@@ -575,22 +543,25 @@ class ClientData:
     icon = ''
     capabilities = ''
     check_last_save = True
-    ignored_extensions = getGitIgnoredExtensions()
-    useless_str = ''
-    useless_int = 0
+    ignored_extensions = GIT_IGNORED_EXTENSIONS
+    template_origin = ''
+    jack_client_name = ''
+    jack_naming = 0
+    ray_hack = None
+    ray_net = None
 
     @staticmethod
     def sisi():
-        return 'sissssissssssissi'
+        return 'sissssissssssisssi'
 
     @staticmethod
-    def newFrom(*args):
+    def new_from(*args):
         client_data = ClientData()
         client_data.update(*args)
         return client_data
 
     @staticmethod
-    def spreadClient(client)->tuple:
+    def spread_client(client)->tuple:
         return (client.client_id, client.protocol,
                 client.executable_path, client.arguments, client.pre_env,
                 client.name, client.prefix_mode, client.custom_prefix,
@@ -598,12 +569,19 @@ class ClientData:
                 client.icon,
                 client.capabilities, int(client.check_last_save),
                 client.ignored_extensions,
-                client.useless_str, client.useless_int)
+                client.template_origin,
+                client.jack_client_name, client.jack_naming)
 
     def gui_init(self, client_id, protocol):
         self.client_id = client_id
         self.protocol = protocol
 
+    def set_ray_hack(self, ray_hack):
+        self.ray_hack = ray_hack
+
+    def set_ray_net(self, ray_net):
+        self.ray_net = ray_net
+
     def update(self, client_id, protocol,
                executable, arguments, pre_env,
                name, prefix_mode, custom_prefix,
@@ -611,7 +589,8 @@ class ClientData:
                icon,
                capabilities, check_last_save,
                ignored_extensions,
-               useless_str, useless_int,
+               template_origin,
+               jack_client_name, jack_naming,
                secure=False):
         self.executable_path = str(executable)
         self.arguments = str(arguments)
@@ -624,6 +603,8 @@ class ClientData:
 
         self.check_last_save = bool(check_last_save)
         self.ignored_extensions = str(ignored_extensions)
+        self.template_origin = template_origin
+        self.jack_naming = jack_naming
 
         if secure:
             return
@@ -643,15 +624,24 @@ class ClientData:
                 self.prefix_mode = PrefixMode.SESSION_NAME
 
         self.capabilities = str(capabilities)
+        self.jack_client_name = jack_client_name
 
-    def updateSecure(self, *args):
+    def update_secure(self, *args):
         self.update(*args, secure=True)
 
     def spread(self)->tuple:
-        return ClientData.spreadClient(self)
+        return ClientData.spread_client(self)
+    
+    def prettier_name(self)->str:
+        if self.label:
+            return self.label
+        if (self.protocol != Protocol.RAY_HACK
+                and self.name):
+            return self.name
+        return self.executable_path
 
 
-class RayHack():
+class RayHack:
     config_file = ""
     save_sig = 0
     stop_sig = 15
@@ -666,7 +656,7 @@ class RayHack():
         return 'siiiisi'
 
     @staticmethod
-    def newFrom(*args):
+    def new_from(*args):
         ray_hack = RayHack()
         ray_hack.update(*args)
         return ray_hack
@@ -694,3 +684,213 @@ class RayHack():
         return (self.config_file, self.save_sig, self.stop_sig,
                 int(self.wait_win), self.no_save_level,
                 self.useless_str, self.useless_int)
+
+
+class RayNet:
+    daemon_url = ''
+    session_root = ''
+    session_template = ''
+    duplicate_state = -1
+    running_daemon_url = ''
+    running_session_root =''
+
+    @staticmethod
+    def sisi():
+        return 'sss'
+
+    @staticmethod
+    def new_from(*args):
+        ray_net = RayNet()
+        ray_net.update(*args)
+        return ray_net
+
+    def update(self, daemon_url, session_root, session_template):
+        self.daemon_url = daemon_url
+        self.session_root = session_root
+        self.session_template = session_template
+
+    def spread(self)->tuple:
+        return (self.daemon_url, self.session_root, self.session_template)
+
+
+class GroupPosition:
+    port_types_view = GROUP_CONTEXT_AUDIO | GROUP_CONTEXT_MIDI
+    group_name = ''
+    null_zone = ''
+    in_zone = ''
+    out_zone = ''
+    null_xy = (0, 0)
+    in_xy = (0, 0)
+    out_xy = (0, 0)
+    flags = 0
+    fully_set = True
+    
+    @staticmethod
+    def get_attributes():
+        return ('port_types_view', 'group_name',
+                'null_zone', 'in_zone', 'out_zone',
+                'null_xy', 'in_xy', 'out_xy', 'flags')
+    
+    @staticmethod
+    def sisi():
+        return 'issssiiiiiii'
+    
+    @staticmethod
+    def new_from(*args):
+        group_position = GroupPosition()
+        group_position.update(*args)
+        return group_position
+    
+    def write_from_dict(self, input_dict: dict):
+        for attr in input_dict:
+            if not attr in self.get_attributes():
+                sys.stderr.write(
+                    'group position has no attribute %s\n' % attr)
+                continue
+            
+            value = input_dict[attr]
+            attr_type = type(value)
+            if attr in ('port_types_view', 'flags'):
+                if attr_type != int:
+                    continue
+            elif attr in ('group_name', 'null_zone', 'in_zone', 'out_zone'):
+                if attr_type != str:
+                    continue
+            elif attr in ('null_xy', 'in_xy', 'out_xy'):
+                if attr_type not in (list, tuple):
+                    continue
+                value = tuple(value)
+            
+            self.__setattr__(attr, value)
+    
+    def is_same(self, other)->bool:
+        if (self.port_types_view == other.port_types_view
+                and self.group_name == other.group_name):
+            return True
+        
+        return False
+    
+    def update(self, port_types_view: int, group_name: str,
+               null_zone: str, in_zone: str, out_zone: str,
+               null_x: int, null_y: int, in_x: int, in_y: int,
+               out_x: int, out_y: int, flags: int):
+        for string in (group_name, null_zone, in_zone, out_zone):
+            if type(string) != str:
+                return 
+        
+        for digit in (port_types_view, null_x, null_y, in_x, in_y,
+                      out_x, out_y, flags):
+            if type(digit) == int:
+                continue
+            
+            if type(digit) != str:
+                return
+            
+            if (digit.isdigit()
+                    or (digit.startswith('-')
+                        and digit.replace('-', '', 1).isdigit())):
+                continue
+            else:
+                return
+
+        self.port_types_view = port_types_view
+        self.group_name = group_name
+        self.null_zone = null_zone
+        self.in_zone = in_zone
+        self.out_zone = out_zone
+        self.null_xy = (int(null_x), int(null_y))
+        self.in_xy = (int(in_x), int(in_y))
+        self.out_xy = (int(out_x), int(out_y))
+        self.flags = int(flags)
+        
+    def spread(self)->tuple:
+        return (self.port_types_view, self.group_name,
+                self.null_zone, self.in_zone, self.out_zone,
+                self.null_xy[0], self.null_xy[1], self.in_xy[0], self.in_xy[1],
+                self.out_xy[0], self.out_xy[1], self.flags)
+    
+    def to_dict(self)->dict:
+        new_dict = {}
+        
+        for attr in self.__dir__():
+            if attr in self.get_attributes():
+                new_dict[attr] = self.__getattribute__(attr)
+        
+        return new_dict
+    
+    def get_str_value(self, attr: str)->str:
+        if attr not in self.get_attributes():
+            return ''
+        
+        return str(self.__getattribute__(attr))
+    
+class PortGroupMemory:
+    group_name = ''
+    port_type = 0
+    port_mode = 0
+    port_names = []
+    above_metadatas = False
+    
+    @staticmethod
+    def get_attributes():
+        return ('group_name', 'port_type', 'port_mode',
+                'above_metadatas', 'port_names')
+    
+    @staticmethod
+    def new_from(*args):
+        portgrp_memory = PortGroupMemory()
+        portgrp_memory.update(*args)
+        return portgrp_memory
+    
+    def write_from_dict(self, input_dict: dict):
+        for attr in input_dict:
+            if not attr in self.get_attributes():
+                sys.stderr.write(
+                    'PortGroupMemory has no attribute %s\n' % attr)
+                continue
+
+            value = input_dict[attr]
+            attr_type = type(value)
+            if value == 'group_name' and attr_type != str:
+                continue
+            if value in ('port_type', 'port_mode') and attr_type != int:
+                continue
+            if value == 'above_metadatas' and attr_type != bool:
+                continue
+            if value == 'port_names' and attr_type not in (tuple, list):
+                continue
+
+            self.__setattr__(attr, value)
+    
+    def update(self, group_name: str, port_type: int, port_mode: int,
+               above_metadatas: int, *port_names):
+        self.group_name = group_name
+        self.port_type = port_type
+        self.port_mode = port_mode
+        self.above_metadatas = bool(above_metadatas)
+        self.port_names = port_names
+    
+    def spread(self)->tuple:
+        return (self.group_name, self.port_type, self.port_mode,
+                int(self.above_metadatas), *self.port_names)
+    
+    def to_dict(self)->dict:
+        new_dict = {}
+        
+        for attr in self.__dir__():
+            if attr in self.get_attributes():
+                new_dict[attr] = self.__getattribute__(attr)
+        
+        return new_dict
+    
+    def has_a_common_port_with(self, other)->bool:
+        if (self.port_type != other.port_type
+                or self.port_mode != other.port_mode
+                or self.group_name != other.group_name):
+            return False
+        
+        for port_name in self.port_names:
+            if port_name in other.port_names:
+                return True
+        
+        return False
