@@ -13,6 +13,7 @@ from gui_tools import ArgParser, CommandLineArgs, init_gui_tools, get_code_root
 from gui_server_thread import GuiServerThread
 from gui_session import SignaledSession
 import ray
+import resources_rc
 
 
 def signal_handler(sig, frame):
@@ -69,7 +70,7 @@ if __name__ == '__main__':
 
     # Add raysession/src/bin to $PATH
     # to can use raysession after make, whitout install
-    ray.addSelfBinToPath()
+    ray.add_self_bin_to_path()
 
     #connect signals
     signal.signal(signal.SIGINT, signal_handler)
